////////////////////////////////////////////////////////////////////////////////
//                            Appearance.h                                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef APPEARANCE_H
#define APPEARANCE_H

#include "X3DTypes.h"
#include "X3DAppearanceNode.h"

namespace X3DTK {

class X3DMaterialNode;
class X3DTextureNode;
class X3DTextureTransformNode;
class LineProperties;
class FillProperties;

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/shape.html">Shape 
/// component</a> defining an Appearance
/// (<a href="../../doc/X3D_Specs/Part01/components/Shape.html#Appearance">
/// X3D documentation</a>).

class Appearance : public X3DAppearanceNode
{
public:
  /// Constructor.
  Appearance();
  /// Constructs a Material from its attributes.
  Appearance(const X3DMaterialNode *material, 
             const X3DTextureNode *texture, 
	     const X3DTextureTransformNode *textureTransform, 
	     const LineProperties *lineProperties, 
	     const FillProperties *fillProperties);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Appearance();
  
  /// Sets the Material attribute.
  void setMaterial(const X3DMaterialNode *material);
  /// Sets the Texture attribute.
  void setTexture(const X3DTextureNode *texture);
  /// Sets the TextureTransform attribute.
  void setTextureTransform(const X3DTextureTransformNode *textureTransform);
  /// Sets the LineProperties attribute.
  void setLineProperties(const LineProperties *lineProperties);
  /// Sets the FillProperties attribute.
  void setFillProperties(const FillProperties *fillProperties);

  /// Gets the Material attribute.  
  inline SFNode getMaterial() const {return material_;};
  /// Gets the Texture attribute.  
  inline SFNode getTexture() const {return texture_;};
  /// Gets the TextureTransform attribute.  
  inline SFNode getTextureTransform() const {return textureTransform_;};
  /// Gets the LineProperties attribute.  
  inline SFNode getLineProperties() const {return lineProperties_;};
  /// Gets the FillProperties attribute.  
  inline SFNode getFillProperties() const {return fillProperties_;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  bool addChild(const SFNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  bool setChild(const SFNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  bool removeChild(const SFNode &N);

protected:  
  /// Copy constructor. 	     
  Appearance(const Appearance &A);
  
  /// Removes the sceneList to the children.
  virtual void removeScenesToChildren(const MFScene &sceneList);
  /// Adds the sceneList to the children.
  virtual void addScenesToChildren(const MFScene &sceneList);
  
private:
  SFNode material_;
  SFNode texture_;
  SFNode textureTransform_;
  SFNode lineProperties_;
  SFNode fillProperties_;
};

}

#endif
