////////////////////////////////////////////////////////////////////////////////
//                            Box.h                                           //
////////////////////////////////////////////////////////////////////////////////

#ifndef BOX_H
#define BOX_H

#include "X3DTypes.h"
#include "X3DGeometry3DNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/geometry3D.html">
/// Geometry3D component</a> defining a Box 
/// (<a href="../../doc/X3D_Specs/Part01/components/geometry3D.html#Box">
/// X3D documentation</a>).

class Box : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Box();
  /// Constructs a Box from \a size, the vector representing the diagonal of the Box. 
  /// The Box is centered on (0, 0, 0).
  Box(const SFVec3f &size);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Box();
  
  /// Sets the size attribute.
  void setSize(const SFVec3f &size);
  
  /// Gets the size attribute.
  inline SFVec3f getSize() const {return size_;};
  
  /// Loads the attributes from a X3DFileElement e.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.
  Box(const Box &B);
  
private:
  SFVec3f size_;
};

}

#endif
