////////////////////////////////////////////////////////////////////////////////
//                            Coordinate.h                                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef COORDINATE_H
#define COORDINATE_H

#include "X3DTypes.h"
#include "X3DCoordinateNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining a Coordinate
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#Coordinate">
/// X3D documentation</a>).

class Coordinate : public X3DCoordinateNode
{
public:
  /// Constructor.
  Coordinate();
  /// Constructs a Coordinate from its attributes.
  Coordinate(const MFVec3f &point);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Coordinate();
  
  /// Sets the Point attribute.
  void setPoint(const MFVec3f &point);
    
  /// Gets the Point attribute.
  inline MFVec3f &getPoint() const {return (MFVec3f &)point_;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;
  
protected: 
  /// Copy constructor.
  Coordinate(const Coordinate &C);
 
private:
  MFVec3f point_;  
};

}

#endif
