////////////////////////////////////////////////////////////////////////////////
//                            FillProperties.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef FILLPROPERTIES_H
#define FILLPROPERTIES_H

#include "X3DTypes.h"
#include "X3DAppearanceChildNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/shape.html">Shape 
/// component</a> defining a FillProperties
/// (<a href="../../doc/X3D_Specs/Part01/components/Shape.html#FillProperties">
/// X3D documentation</a>).

class FillProperties : public X3DAppearanceChildNode
{
public:
  /// Constructor.
  FillProperties();
  /// Constructs a Material from its attributes.
  FillProperties(SFString fillStyle, 
                 SFInt32 hatchStyle, 
		 SFColor hatchColor);
  /// Clones the node.
  virtual SFNode clone() const;		 
  
  /// Destructor.
  ~FillProperties();

  /// Sets the FillStyle attribute.
  void setFillStyle(SFString fillStyle);
  /// Sets the HatchStyle attribute.
  void setHatchStyle(SFInt32 hatchStyle);
  /// Sets the HatchColor attribute.
  void setHatchColor(SFColor hatchColor);
  
  /// Sets the FillStyle attribute.
  inline SFString getFillStyle() const {return fillStyle_;};
  /// Sets the HatchStyle attribute.
  inline SFInt32 getHatchStyle() const {return hatchStyle_;};
  /// Sets the HatchColor attribute.
  inline SFColor getHatchColor() const {return hatchColor_;};

  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor. 
  FillProperties(const FillProperties &F);

private:
  SFString fillStyle_;
  SFInt32 hatchStyle_;
  SFColor hatchColor_;
};

}

#endif
