////////////////////////////////////////////////////////////////////////////////
//                            GLAppearance.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLAPPEARANCE_H
#define GLAPPEARANCE_H

#include "X3DGLAppearanceNode.h"

namespace X3DTK {

class GLMaterial;
class X3DGLTextureNode;
class X3DGLTextureTransformNode;
class GLLineProperties;
class GLFillProperties;


/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// Appearance.

class GLAppearance : public X3DGLAppearanceNode
{
public:
  /// Constructor.
  GLAppearance();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~GLAppearance();
  
  /// Sets the Material attribute.
  void setMaterial(const GLMaterial *material);
  /// Sets the Texture attribute.
  void setTexture(const X3DGLTextureNode *texture);
  /// Sets the TextureTransform attribute.
  void setTextureTransform(const X3DGLTextureTransformNode *textureTransform);
  /// Sets the LineProperties attribute.
  void setLineProperties(const GLLineProperties *lineProperties);
  /// Sets the FillProperties attribute.
  void setFillProperties(const GLFillProperties *fillProperties);
  
  /// Gets the Material attribute.
  inline SFNode getMaterial() {return _material;};
  /// Gets the Texture attribute.
  inline SFNode getTexture() {return _texture;};
  /// Gets the TextureTransform attribute.
  inline SFNode getTextureTransform() {return _textureTransform;};
  /// Gets the LineProperties attribute.
  inline SFNode getLineProperties() {return _lineProperties;};
  /// Gets the FillProperties attribute.
  inline SFNode getFillProperties() {return _fillProperties;};
   
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFNode &N); 

protected:  
  /// Copy constructor.	     
  GLAppearance(const GLAppearance &A);
  
private:
  SFNode _material;
  SFNode _texture;
  SFNode _textureTransform;
  SFNode _lineProperties;
  SFNode _fillProperties;
};

}

#endif
