////////////////////////////////////////////////////////////////////////////////
//                            GLIndexedFaceSet.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLINDEXEDFACESET_H
#define GLINDEXEDFACESET_H

#include "X3DGLComposedGeometryNode.h"

#include <vector>

namespace X3DTK {

class IndexedFaceSet;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// IndexedFaceSet.

class GLIndexedFaceSet : public X3DGLComposedGeometryNode
{
public:
  /// Constructor.
  GLIndexedFaceSet();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~GLIndexedFaceSet();
  
  /// Gets the ccw.
  inline SFBool &getCcw() const {return (SFBool &)_ccw;};
  /// Gets the solid.
  inline SFBool &getSolid() const {return (SFBool &)_solid;};
  /// Gets the verticesDuplicated.
  inline SFBool &getVerticesDuplicated() const {return (SFBool &)_verticesDuplicated;};
  /// Gets the color.
  inline SFBool &getColor() const {return (SFBool &)_color;};
  /// Gets the texCoord.
  inline SFBool &getTexCoord() const {return (SFBool &)_texCoord;};

  /// Gets the _N3F_V3FvertexFaceArray.
  inline std::vector<N3F_V3F> &getN3F_V3FvertexFaceArray() const {return (std::vector<N3F_V3F> &)_N3F_V3FvertexFaceArray;};
  /// Gets the _C4F_N3F_V3FvertexFaceArray.
  inline std::vector<C4F_N3F_V3F> &getC4F_N3F_V3FvertexFaceArray() const {return (std::vector<C4F_N3F_V3F> &)_C4F_N3F_V3FvertexFaceArray;};
  /// Gets the _T2F_N3F_V3FvertexFaceArray.
  inline std::vector<T2F_N3F_V3F> &getT2F_N3F_V3FvertexFaceArray() const {return (std::vector<T2F_N3F_V3F> &)_T2F_N3F_V3FvertexFaceArray;};
  /// Gets the _T2F_C4F_N3F_V3FvertexFaceArray.
  inline std::vector<T2F_C4F_N3F_V3F> &getT2F_C4F_N3F_V3FvertexFaceArray() const {return (std::vector<T2F_C4F_N3F_V3F> &)_T2F_C4F_N3F_V3FvertexFaceArray;};
  /// Gets the indexFaceArray.
  inline std::vector<unsigned int> &getIndexFaceArray() const {return (std::vector<unsigned int> &)_indexFaceArray;};

protected:
  /// Copy constructor.         
  GLIndexedFaceSet(const GLIndexedFaceSet &G);
  
private:
  //GL attributes
  SFBool _ccw;
  SFBool _solid;
  SFBool _verticesDuplicated;
  SFBool _color;
  SFBool _texCoord;
 
  std::vector<N3F_V3F> _N3F_V3FvertexFaceArray;
  std::vector<C4F_N3F_V3F> _C4F_N3F_V3FvertexFaceArray;
  std::vector<T2F_N3F_V3F> _T2F_N3F_V3FvertexFaceArray;
  std::vector<T2F_C4F_N3F_V3F> _T2F_C4F_N3F_V3FvertexFaceArray;
  std::vector<unsigned int> _indexFaceArray;
};

}

#endif
