////////////////////////////////////////////////////////////////////////////////
//                            GLMaterial.h                                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLMATERIAL_H
#define GLMATERIAL_H

#include "X3DGLMaterialNode.h"

#include <vector>

namespace X3DTK {

class Material;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// Material.

class GLMaterial : public X3DGLMaterialNode
{
public:
  /// Constructor.
  GLMaterial();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~GLMaterial();

  /// Gets the diffuseColor.
  inline MFFloat &getDiffuseColor() const {return (MFFloat &)_diffuseColor;};
  /// Gets the ambientColor.
  inline MFFloat &getAmbientColor() const {return (MFFloat &)_ambientColor;};
  /// Gets the emissiveColor.
  inline MFFloat &getEmissiveColor() const {return (MFFloat &)_emissiveColor;};
  /// Gets the shininess.
  inline SFFloat &getShininess() const {return (SFFloat &)_shininess;};
  /// Gets the specularColor.
  inline MFFloat &getSpecularColor() const {return (MFFloat &)_specularColor;};

protected:    
  /// Copy constructor.	   
  GLMaterial(const GLMaterial &M);
  
private:
  //GL attributes
  MFFloat _diffuseColor;
  MFFloat _ambientColor;
  MFFloat _emissiveColor;
  SFFloat _shininess;
  MFFloat _specularColor;
};

}

#endif
