////////////////////////////////////////////////////////////////////////////////
//                            GLTriangleFanSet.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLTRIANGLEFANSET_H
#define GLTRIANGLEFANSET_H

#include "X3DGLComposedGeometryNode.h"

#include <vector>

namespace X3DTK {

class TriangleFanSet;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// TriangleFanSet.

class GLTriangleFanSet : public X3DGLComposedGeometryNode
{
public:
  /// Constructor.
  GLTriangleFanSet();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~GLTriangleFanSet();

  /// Gets the ccw.
  inline SFBool &getCcw() const {return (SFBool &)_ccw;};
  /// Gets the solid.
  inline SFBool &getSolid() const {return (SFBool &)_solid;};
  /// Gets the verticesDuplicated.
  inline SFBool &getVerticesDuplicated() const {return (SFBool &)_verticesDuplicated;};
  /// Gets the color.
  inline SFBool &getColor() const {return (SFBool &)_color;};
  /// Gets the texCoord.
  inline SFBool &getTexCoord() const {return (SFBool &)_texCoord;};

  /// Gets the _N3F_V3FvertexFaceArray.
  inline std::vector<std::vector<N3F_V3F> *> &getN3F_V3FvertexFanArrayArray() const {return (std::vector<std::vector<N3F_V3F> *> &)_N3F_V3FvertexFanArrayArray;};
  /// Gets the _C4F_N3F_V3FvertexFaceArray.
  inline std::vector<std::vector<C4F_N3F_V3F> *> &getC4F_N3F_V3FvertexFanArrayArray() const {return (std::vector<std::vector<C4F_N3F_V3F> *> &)_C4F_N3F_V3FvertexFanArrayArray;};

protected:
  /// Copy constructor.    
  GLTriangleFanSet(const GLTriangleFanSet &T);
    
private:
  //GL attributes
  SFBool _ccw;
  SFBool _solid;
  SFBool _verticesDuplicated;
  SFBool _color;
  SFBool _texCoord;
  
  std::vector<std::vector<C4F_N3F_V3F> *> _C4F_N3F_V3FvertexFanArrayArray;  
  std::vector<std::vector<N3F_V3F> *> _N3F_V3FvertexFanArrayArray;  
};

}

#endif
