////////////////////////////////////////////////////////////////////////////////
//                            LineProperties.h                               //
////////////////////////////////////////////////////////////////////////////////

#ifndef LINEPROPERTIES_H
#define LINEPROPERTIES_H

#include "X3DTypes.h"
#include "X3DAppearanceChildNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/shape.html">Shape 
/// component</a> defining a LineProperties
/// (<a href="../../doc/X3D_Specs/Part01/components/Shape.html#LineProperties">
/// X3D documentation</a>).

class LineProperties : public X3DAppearanceChildNode
{
public:
  /// Constructor.
  LineProperties();
  /// Constructs a Material from its attributes.
  LineProperties(SFInt32 lineStyle, 
                 SFFloat width);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor. 
  ~LineProperties();
  
  /// Sets the LineStyle attribute.
  void setLineStyle(SFInt32 lineStyle);
  /// Sets the Width attribute.
  void setWidth(SFFloat width);
  
  /// Gets the LineStyle attribute.
  inline SFInt32 getLineStyle() const {return lineStyle_;};
  /// Gets the Width attribute.
  inline SFFloat getWidth() const {return width_;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:  
  /// Copy constructor.		 
  LineProperties(const LineProperties &L);
    
private:
  SFInt32 lineStyle_;
  SFFloat width_;
};

}

#endif
