////////////////////////////////////////////////////////////////////////////////
//                            Material.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef MATERIAL_H
#define MATERIAL_H

#include "X3DTypes.h"
#include "X3DMaterialNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/shape.html">Shape 
/// component</a> defining a Material
/// (<a href="../../doc/X3D_Specs/Part01/components/Shape.html#Material">
/// X3D documentation</a>).

class Material : public X3DMaterialNode
{
public:
  /// Constructor.
  Material();
  /// Constructs a Material from its attributes.
  Material(SFFloat ambientIntensity, 
           SFColor diffuseColor, 
	   SFColor emissiveColor, 
	   SFFloat shininess, 
	   SFColor specularColor, 
	   SFFloat transparency);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Material();
  
  /// Sets the AmbientIntensity attribute.
  void setAmbientIntensity(SFFloat ambientIntensity);
  /// Sets the DiffuseColo attribute.
  void setDiffuseColor(SFColor diffuseColor);
  /// Sets the EmissiveColo attribute.
  void setEmissiveColor(SFColor emissiveColor);
  /// Sets the Shininess attribute.
  void setShininess(SFFloat shininess);  
  /// Sets the SpecularColor attribute.
  void setSpecularColor(SFColor specularColor);  
  /// Sets the Transparenc attribute.
  void setTransparency(SFFloat transparency);  
  
  /// Gets the AmbientIntensity attribute.
  inline SFFloat getAmbientIntensity() const {return ambientIntensity_;};       
  /// Gets the DiffuseColor attribute.
  inline SFColor getDiffuseColor() const {return diffuseColor_;};     
  /// Gets the EmissiveColor attribute.
  inline SFColor getEmissiveColor() const {return emissiveColor_; };    
  /// Gets the Shininess attribute.
  inline SFFloat getShininess() const {return shininess_;};        
  /// Gets the SpecularColor attribute.
  inline SFColor getSpecularColor() const {return specularColor_;};    
  /// Gets the Transparency attribute.
  inline SFFloat getTransparency() const {return transparency_;};   
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor. 
  Material(const Material &M);
    
private:
  SFFloat ambientIntensity_;       
  SFColor diffuseColor_;     
  SFColor emissiveColor_;    
  SFFloat shininess_;        
  SFColor specularColor_;    
  SFFloat transparency_;     
};

}

#endif
