////////////////////////////////////////////////////////////////////////////////
//                            SFString.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef SFSTRING_H
#define SFSTRING_H

#include <string>

namespace X3DTK {

/// Class providing a string derived from std::string. Functions are added like the 
/// different conversions.

class SFString : public std::string
{
public:
  /// Default constructor.
  SFString();
  /// Conversion from char *.
  SFString(const char *s);
  /// Conversion from std::string.
  SFString(const std::string &s);
  /// Copy constructor.
  SFString(const SFString &s);
  
  /// Conversion to char *
  operator const char *() const;
  /// Conversion to int.  
  int toInt() const;
  /// Conversion to unsigned int.  
  unsigned int toUint() const;
  /// Conversion to float.  
  float toFloat() const;
  /// Conversion to double.  
  double toDouble() const;
  /// Returns a string that is the string converted to lowercase.
  SFString lower() const;
  /// Returns a string that is the string converted to uppercase.
  SFString upper() const;
  /// Returns the path of the string supposed to be a file.
  SFString getPath() const;
  
  /// Converts a floating number to a SFString.  
  static SFString number(float f);
  /// Converts a double number to a SFString.
  static SFString number(double d);
  /// Converts an integer to a SFString.
  static SFString number(int i); 
  /// Converts an unsigned integer to a SFString.
  static SFString number(unsigned int ui); 
};

}

#endif
