////////////////////////////////////////////////////////////////////////////////
//                            SFVec3f.h                                       //
////////////////////////////////////////////////////////////////////////////////

#ifndef VECTOR3F_H
#define VECTOR3F_H

#include <math.h>
#include <iostream>

namespace X3DTK {

class SFString;
class SFPoint3f;

/// Class providing an implementation of a 3D float vector. 
/// SFVec3f is a vector in an affin space, which different from a point represented
/// by the SFPoint3f class.

class SFVec3f
{
public:
  /// x coordinates.
  float x;
  /// y coordinates.
  float y;
  /// z coordinates.
  float z;

  /// Default constructor.
  SFVec3f();
  /// Constructor with x, y and z coordinates in parameters.
  SFVec3f(float x, float y, float z);
  /// Copy constructor
  SFVec3f(const SFVec3f &v);
  /// Overloaded assignment operator.
  SFVec3f &operator= (SFVec3f v);
  /// Explicit constructor from a string of which format is "x y z".
  explicit SFVec3f(const SFString &s); 
  /// Conversion from point to vector.
  explicit SFVec3f(const SFPoint3f &P);
  
  /// Overloaded operator==.
  friend bool operator== (const SFVec3f &v1, const SFVec3f &v2);
  /// Overloaded operator!=.
  friend bool operator!= (const SFVec3f &v1, const SFVec3f &v2);
  /// Overloaded operator+.
  friend SFVec3f operator+ (const SFVec3f &v1, const SFVec3f &v2);
  /// Overloaded operator-.
  friend SFVec3f operator- (const SFVec3f &v1, const SFVec3f &v2);
  /// Overloaded operator*.
  friend float operator* (const SFVec3f &v1, const SFVec3f &v2);
  /// Overloaded operator*.
  friend SFVec3f operator* (const float a, const SFVec3f &v);
  /// Cross product between two vectors.
  friend SFVec3f crossprod(const SFVec3f &v1, const SFVec3f &v2);
  /// Returns the norm of the vector.
  float norm() const;
  /// Normalizes the vector.
  SFVec3f normalize();
  /// Returns the vector normalized but doesn't change the vector.
  SFVec3f normalized() const;
  /// Returns the pointed array.
  inline float *f_data() {return &x;};
};

}

#endif
