////////////////////////////////////////////////////////////////////////////////
//                            SimpleX3DGLScene.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef SIMPLEX3DGLSCENE_H
#define SIMPLEX3DGLSCENE_H

#include "X3DTypes.h"
#include "Bbox.h"

namespace X3DTK {

class X3DLoader;
class X3DBboxUpdater;
class GLConverter;
class X3DGLCreator;
class X3DGLUpdater;
class X3DGLRenderer;
class X3DMemReleaser;
class Scene;
class GLScene;

/// Class defining a facade to allow an easy load and display of an X3D scene.

class SimpleX3DGLScene
{
public:
  /// Constructor.
  SimpleX3DGLScene();
  /// Destructor.
  virtual ~SimpleX3DGLScene();
  
  /// Loads ths scene from file.
  void load(const char *file);
  /// Draw the scene in an opengl context.
  virtual void draw();
  /// Gets the Bounding Box center.
  inline SFVec3f getBboxMin() const {return min;};
  /// Gets the Bounding Box size.
  inline SFVec3f getBboxMax() const {return max;};
  /// Releases the memory.
  void release();

  /// Sets the loader.
  void setLoader(X3DLoader *loader);
  /// Sets the BboxUpdater.
  void setBboxUpdater(X3DBboxUpdater *bboxUpdater);  
  /// Sets the GLCreator.
  void setGLCreator(X3DGLCreator *creator);  
  /// Sets the GLUpdater.
  void setGLUpdater(X3DGLUpdater *glupdater);  
  /// Sets the GLRenderer.
  void setGLRenderer(X3DGLRenderer *renderer);  
  /// Sets the MemReleaser.
  void setMemReleaser(X3DMemReleaser *memReleaser);  
  
protected:
  X3DLoader *_loader;
  X3DBboxUpdater *_bboxupdater;
  GLConverter *_converter;
  X3DGLRenderer *_renderer;
  X3DMemReleaser *_releaser;
  Scene *s;
  GLScene *gs;
  SFVec3f min;
  SFVec3f max;
};

}

#endif
