////////////////////////////////////////////////////////////////////////////////
//                            Sphere.h                                        //
////////////////////////////////////////////////////////////////////////////////

#ifndef SPHERE_H
#define SPHERE_H

#include "X3DTypes.h"
#include "X3DGeometry3DNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/geometry3D.html">
/// Geometry3D component</a> defining a Sphere
/// (<a href="../../doc/X3D_Specs/Part01/components/geometry3D.html#Sphere">
/// X3D documentation</a>).

class Sphere : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Sphere();
  /// Constructor from its attribute.
  explicit Sphere(SFFloat radius);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Sphere();
  
  /// Sets the radius attribute.
  void setRadius(SFFloat radius);
  
  /// Gets the radius attribute.
  inline SFFloat getRadius() const {return radius_;};
  
  /// Loads the attributes from a X3DFileElement e.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.
  Sphere(const Sphere &C);

private:
  SFFloat radius_;
};

}

#endif
