////////////////////////////////////////////////////////////////////////////////
//                            SphereDrawArray.h                               //
////////////////////////////////////////////////////////////////////////////////

#ifndef SPHEREARRAY_H
#define SPHEREARRAY_H

#include "X3DTypes.h"
#include "X3DGLTypes.h"

#include <vector>

namespace X3DTK {

/// Class providing the arrays required by the opengl command drawElements() to draw
/// a Sphere.

class SphereDrawArray
{
public:
  /// Gets the CylinderDrawArray of section section.
  static SphereDrawArray *getInstanceOfSection(unsigned int section);
  /// Removes the CylinderDrawArray of section section.
  void removeInstance();
  
  
  /// Gets the number of vertices of the sphere.
  unsigned int getSphereSize() const;
  /// Gets the address of the first element of the vertex array.
  const void *getSphereVertexArrayAddress() const;
  /// Gets the address of the first index of the index array. 
  const unsigned int *getSphereIndexArrayAddress() const;
  
private:
  unsigned int _section;

  /// Constructs the arrays for a sphere of sub_theta subdivisions for the theta angle 
  /// and sub_phi subdivisions for the phi angle
  SphereDrawArray(unsigned int section);

  typedef struct 
  {
    unsigned int count;
    SphereDrawArray *ref;
  } data;
  
  typedef std::list<std::pair<unsigned int, data> > refList;

  static refList _refList;

  std::vector<N3F_V3F> _sphereVertexArray;
  std::vector<unsigned int> _sphereIndexArray;
};

}

#endif
