////////////////////////////////////////////////////////////////////////////////
//                            Transform.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef TRANSFORM_H
#define TRANSFORM_H

#include "X3DTypes.h"
#include "X3DGroupingNode.h"
#include "X3DBoundedObject.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/group.html">
/// Grouping component</a> defining a Transform
/// (<a href="../../doc/X3D_Specs/Part01/components/grouping.html#Transform">
/// X3D documentation</a>).

class Transform : public X3DGroupingNode, public X3DBoundedObject
{
public:  
  /// Constructor.
  Transform();
  /// Constructs a Transform from its attributes.
  Transform(const SFVec3f &center, 
	    const SFRotation &rotation, 
	    const SFVec3f &scale, 
	    const SFRotation &scaleOrientation, 
	    const SFVec3f &translation, 
	    const SFVec3f &bboxCenter, 
	    const SFVec3f &bboxSize);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Transform();

  /// Sets the center of the transformation.
  void setCenter(const SFVec3f &center);
  /// Sets the rotation of the transformation.
  void setRotation(const SFRotation &rotation);
  /// Sets the scale of the transformation.
  void setScale(const SFVec3f &scale);
  /// Sets the scale orientation of the transformation.
  void setScaleOrientation(const SFRotation &scaleOrientation);
  /// Sets the translation of the transformation.
  void setTranslation(const SFVec3f &translation);

  /// Gets the center of the transformation.
  inline SFVec3f getCenter() const {return center_;};
  /// Gets the rotation of the transformation.
  inline SFRotation getRotation() const {return rotation_;};
  /// Gets the scale of the transformation.
  inline SFVec3f getScale() const {return scale_;};
  /// Gets the scale orientation of the transformation.
  inline SFRotation getScaleOrientation() const {return scaleOrientation_;};
  /// Gets the translation of the transformation.
  inline SFVec3f getTranslation() const {return translation_;};

  /// Loads the attributes from a X3DFileElement *.
  void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.
  Transform(const Transform &T);

private:
  SFVec3f center_;
  SFRotation rotation_;
  SFVec3f scale_;
  SFRotation scaleOrientation_;
  SFVec3f translation_;
};

}

#endif
