////////////////////////////////////////////////////////////////////////////////
//                            TriangleSet.h                                   //
////////////////////////////////////////////////////////////////////////////////

#ifndef TRIANGLESET_H
#define TRIANGLESET_H

#include "X3DTypes.h"
#include "X3DComposedGeometryNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining a TriangleSet
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#TriangleSet">
/// X3D documentation</a>).

class TriangleSet : public X3DComposedGeometryNode
{
public:
  /// Constructor.
  TriangleSet();
  /// Constructs a triangleFanSet from its attributes.
  TriangleSet(const X3DColorNode *color,
              const X3DCoordinateNode *coord,
              const X3DNormalNode *normal,
              const X3DTextureCoordinateNode *texCoord,
              SFBool ccw,
              SFBool colorPerVertex,
              SFBool convex,
              SFFloat creaseAngle,
              SFBool normalPerVertex,
              SFBool solid);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~TriangleSet();
  
  /// Sets the Convex attribute.
  void setConvex(SFBool convex);
  /// Sets the CreaseAngle attribute.
  void setCreaseAngle(SFFloat creaseAngle);
  /// Sets the NormalPerVertex attribute.
  
  /// Gets the Convex attribute.
  inline SFBool getConvex() const {return convex_;};
  /// Gets the CreaseAngle attribute.
  inline SFFloat getCreaseAngle() const {return creaseAngle_;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;
  
protected: 
  /// Copy constructor.       
  TriangleSet(const TriangleSet &T);
  
private:
  SFBool convex_;  
  SFFloat creaseAngle_; 
};

}

#endif
