////////////////////////////////////////////////////////////////////////////////
//                            X3DAbstractNode.h                               //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DABSTRACTNODE_H
#define X3DABSTRACTNODE_H

#include "X3DTypes.h"
#include "X3DFileElement.h"
#include "Type.h"
#include "NameDict.h"

namespace X3DTK {

class Scene;
class FileElement;

/// X3DAbstractNode defines the root class of the derivation tree.

class X3DAbstractNode
{
public:
  /// Constructor.
  X3DAbstractNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DAbstractNode();

  /// Gets the name of the node.
  //inline SFString getName() const {return DEFDict.getNameOfNode((X3DAbstractNode *)this);};
  /// Gets the type name of the node.
  inline SFString getTypeName() const {return _type->getName();};
  /// Gets the type of the node.
  inline Type *getType() const {return _type;};
  /// Gets the Scene of the node.
  inline MFScene getSceneList() const {return _sceneList;};
  /// Gets the parent list.
  inline MFNode getParentList() const {return _parentList;};
      
  /// Loads the attributes from a FileElement.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;
    
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFNode &N);
  /// Removes the child if exists. Returns TRUE if suceeds.
  virtual bool removeChild(const SFNode &N);

  static NameDict DEFDict;
    
protected:
  /// Copy constructor.
  X3DAbstractNode(const X3DAbstractNode &N);
  
  /// Removes the scenes.
  static void removeScenes(SFNode child, const MFScene &sceneList);
  /// Removes the scenes to the children.
  virtual void removeScenesToChildren(const MFScene &sceneList);
  /// Adds the scenes.
  static void addScenes(SFNode child, const MFScene &sceneList);
  /// Adds the scenes to the children.
  virtual void addScenesToChildren(const MFScene &sceneList);

  /// Remove link between A and B.
  static void removeLink(SFNode father, SFNode child);
  /// Adds link between A and B.
  static void addLink(SFNode father, SFNode child);

  /// Method to call at the first line of the body of the constructor, and passing 
  /// in argument the name of the tag of the X3D file.
  inline void defineTagName(const SFString &name, const SFString &component = SFString("")) {Type::defineTagName(&_type, name, component);};

  MFScene _sceneList;
  
private:
  // Adds a parent.
  void addParent(const SFNode &N);
  // Removes a parent.
  bool removeParent(const SFNode &N);
   
  Type *_type;
  MFNode _parentList;
};

}

#endif
