////////////////////////////////////////////////////////////////////////////////
//                            X3DComponent.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DCOMPONENT_H
#define X3DCOMPONENT_H

#include "X3DTypes.h"
#include "Component.h"

namespace X3DTK {

/// Abstract class defining the concept of component.

class X3DComponent
{
public:
  /// Constructor.
  X3DComponent();
  /// Destructor.
  virtual ~X3DComponent() = 0;
  
  /// Returns the component.
  inline Component *getComponent() const {return component;};
  /// Returns the name of the component.
  inline SFString getName() const {return component->getName();};
  
  /// Adds one proxy.
  void addOneProxy();
  /// Removes one proxy.
  void removeOneProxy();
  
  /// Returns the number of proxies.
  inline unsigned int getProxyNumber() const {return _proxyNumber;};
  
protected:
  void defineComponentName(const SFString &name);
  Component *component;

private:  
  unsigned int _proxyNumber;  
};

}

#endif
