////////////////////////////////////////////////////////////////////////////////
//                            X3DComponentWalker.h                            //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DCOMPONENTWALKER_H
#define X3DCOMPONENTWALKER_H

#include "X3DComponent.h"
#include "WalkingFunction.h"
#include "NodeCreationProxy.h"
#include "X3DAbstractNode.h"

namespace X3DTK {

class NodeWalkingProxy;
class NodeVisitingProxy;

/// Abstract class defining the concept of component walker.

class X3DComponentWalker : public X3DComponent
{
public:
  /// Constructor.
  X3DComponentWalker();
  /// Destructor.
  virtual ~X3DComponentWalker() = 0;
  
  /// Sets the NodeVisitingProxy
  void setNodeWalkingProxy(const NodeWalkingProxy *nodeWalkingProxy);
  /// Sets the NodeVisitingProxy
  void setNodeVisitingProxy(const NodeVisitingProxy *nodeVisitingProxy);
  
  /// Sets the function active.
  template<class T> 
  void setActiveFunction()
  {
    T *t = new T();
    //iterating the list
    SFString name = t->getName();
    delete t;
    //checking if the node belongs to the component.
/*    if ((nodeCreationProxy_ != 0) && (nodeCreationProxy_->getComponentNameOf(name) != name_))
    {
      std::cerr << "warning: " << name << " doesn't belong to the " << name_ << " component!" << std::endl;
      return;
    }*/
    WalkingDict::const_iterator it = walkingDict_.find(name);
    if (it != walkingDict_.end())
      (*it).second->setActive();
      
  };
  /// Sets the function inactive.
  template<class T> 
  void setInactiveFunction()
  {
    T *t = new T();
    //iterating the list
    SFString name = t->getName();
    delete t;
    //checking if the node belongs to the component.
/*    if ((nodeCreationProxy_ != 0) && (nodeCreationProxy_->getComponentNameOf(name) != name_))
    {
      std::cerr << "warning: " << name << " doesn't belong to the " << name_ << " component!" << std::endl;
      return;
    }*/
    if (name != "X3DNode")
    {
      WalkingDict::const_iterator it = walkingDict_.find(name);
      if (it != walkingDict_.end())
        (*it).second->setInactive();
    } 
  };
  /// Sets all the functions active.
  void setActiveAllFunctions();
  /// Sets all the functions inactive.  
  void setInactiveAllFunctions();
  
  /// Gets the walking function of name.
  WalkingFunction *getWalkingFunctionOf(const SFString &name) const;
  
protected:  
  /// Reference to the NodeWalkingProxy.
  NodeWalkingProxy *nodeWalkingProxy;
  /// Reference to the NodeVisitingProxy.
  NodeVisitingProxy *nodeVisitingProxy;
  /// Defines a new creation function.
  template<class C, class T> 
  void defineNewWalkingFunction(void (C::*ptrF)(T *) const)
  {
    Type::beginFunctionDefinition();
    
    WalkingFunction *WF = new WalkingFunction(reinterpret_cast<ptrToWalkingFunction>(ptrF), this);
    //finding if an occurence of T
    T *t = new T();
    SFString name = t->getTypeName();
    Component *TComponent = t->getType()->getComponent();
    //checking if the node belongs to the component.
    if (TComponent != component)
    {
      std::cerr << "warning: defineNewWalkingFunction for " << name << " which doesn't belong to the " << component->getName() << " component," << std::endl;
      std::cerr << "         but to the " << t->getType()->getComponent()->getName() << " component!" << std::endl;
    }         
    else
      walkingDict_.insert(std::pair<SFString, WalkingFunction *>(name, WF));
    
    delete t;       
    Type::endFunctionDefinition();
  };
  
private:
  WalkingDict walkingDict_;
};

}

#endif
