////////////////////////////////////////////////////////////////////////////////
//                            X3DGLGroupingNode.h                             //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef X3DGLGROUPINGNODE
#define X3DGLGROUPINGNODE

#include "X3DGLChildNode.h"

namespace X3DTK {

/// Class providing an interface for the equivalent of the X3DGroupingNode in the X3DGL tree.

class X3DGLGroupingNode : public X3DGLChildNode
{
public:
  /// Constructor.
  X3DGLGroupingNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DGLGroupingNode();

  /// Adds a child to the Children attribute. Return TRUE if succeeds.
  virtual bool addChild(const SFNode &N);
  /// Sets the child. Behaves like addChild.
  virtual bool setChild(const SFNode &N);
  /// Removes the child. Return TRUE if succeeds.
  virtual bool removeChild(const SFNode &N);
  
  /// Gets the Children attribute.
  inline MFNode getChildren() const {return _children;};

protected:
  /// Copy constructor.		  
  X3DGLGroupingNode(const X3DGLGroupingNode &N);

  /// Removes the sceneList to the children.
  virtual void removeScenesToChildren(const MFScene &sceneList);
  /// Adds the sceneList to the children.
  virtual void addScenesToChildren(const MFScene &sceneList);
  
  /// Children attribute.
  MFNode _children;
};

}

#endif
