////////////////////////////////////////////////////////////////////////////////
//                            X3DGLShapeNode.h                                //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DGLSHAPENODE_H
#define X3DGLSHAPENODE_H

#include "X3DGLChildNode.h"

namespace X3DTK {

class X3DGLGeometryNode;
class X3DGLAppearanceNode;

/// Class providing an interface for the equivalent of the X3DShapeNode in the X3DGL tree.

class X3DGLShapeNode : public X3DGLChildNode
{
public:
  /// Constructor.
  X3DGLShapeNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DGLShapeNode();
  
  /// Sets the Geometry attribute.
  void setGeometry(const X3DGLGeometryNode *geometry);
  /// Sets the Appearance attribute.
  void setAppearance(const X3DGLAppearanceNode *appearance);

  /// Gets the Geometry attribute.
  inline SFNode getGeometry() const {return _geometry;};
  /// Gets the Appearance attribute.
  inline SFNode getAppearance() const {return _appearance;};
  
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFNode &N);
  
protected:
  /// Copy constructor.	       
  X3DGLShapeNode(const X3DGLShapeNode &N);
  
  /// Removes the sceneList to the children.
  virtual void removeScenesToChildren(const MFScene &sceneList);
  /// Adds the sceneList to the children.
  virtual void addScenesToChildren(const MFScene &sceneList);
  
  SFNode _geometry;
  SFNode _appearance;  
};

}

#endif
