////////////////////////////////////////////////////////////////////////////////
//                            X3DGroupingNode.h                               //
//////////////////////////////////////////////////////////////////////////////// 

#ifndef X3DGROUPINGNODE
#define X3DGROUPINGNODE

#include "X3DTypes.h"
#include "X3DChildNode.h"

namespace X3DTK {

/// Base class of the <a href="../../doc/X3D_Specs/Part01/components/group.html">
/// Grouping component</a> defining an X3DGroupingNode
/// (<a href="../../doc/X3D_Specs/Part01/components/grouping.html#X3DGroupingNode">
/// X3D documentation</a>).

class X3DGroupingNode : public X3DChildNode
{
public:
  /// Constructor.
  X3DGroupingNode();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DGroupingNode();

  /// Adds a child to the Children attribute. Returns TRUE if succeeds.
  bool addChild(const SFNode &N);
  /// Sets the child. Behaves like addChild.
  bool setChild(const SFNode &N);
  /// Removes the child. Return TRUE if succeeds.
  bool removeChild(const SFNode &N);
  
  /// Gets the Children attribute.
  inline MFNode getChildren() const {return children;};
  
protected:
  /// Copy constructor.		  
  X3DGroupingNode(const X3DGroupingNode &N);

  /// Removes the sceneList to the children.
  virtual void removeScenesToChildren(const MFScene &sceneList);
  /// Adds the sceneList to the children.
  virtual void addScenesToChildren(const MFScene &sceneList);
  
  /// Children attribute.
  MFNode children;
};

}

#endif
