////////////////////////////////////////////////////////////////////////////////
//                            X3DTextureCoordinateNode.h                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DTEXTURECOORDINATENODE_H
#define X3DTEXTURECOORDINATENODE_H

#include "X3DTypes.h"
#include "X3DGeometricPropertyNode.h"

namespace X3DTK {

/// Base class of the <a href="../../doc/X3D_Specs/Part01/components/texturing.html">
/// Texturing component</a> defining an X3DTextureCoordinateNode
/// (<a href="../../doc/X3D_Specs/Part01/components/texturing.html#X3DTextureCoordinateNode">
/// X3D documentation</a>).

class X3DTextureCoordinateNode : public X3DGeometricPropertyNode
{
public:  
  /// Constructor.
  X3DTextureCoordinateNode();
  /// Constructs an X3DTextureCoordinateNode from its attributes.
  X3DTextureCoordinateNode(const MFVec2f &point);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Virtual destructor.
  virtual ~X3DTextureCoordinateNode();

  /// Sets the Point attribute.
  void setPoint(const MFVec2f &point);
    
  /// Gets the Point attribute.
  inline MFVec2f &getPoint() const {return (MFVec2f &)point;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.	
  X3DTextureCoordinateNode(const X3DTextureCoordinateNode &N);  
  
  /// The point array.
  MFVec2f point;  
};

}

#endif
