/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            NodeCreationProxy.h                             //
////////////////////////////////////////////////////////////////////////////////

#ifndef NODECREATIONPROXY_H
#define NODECREATIONPROXY_H

#include "X3DNodeProxy.h"
#include "Type.h"

namespace X3DTK {

class X3DComponentCreator;

/// Concrete class defining a NodeCreationProxy, creating X3D nodes from their names.

class NodeCreationProxy : public X3DNodeProxy
{
public:  
  /// Constructor.
  NodeCreationProxy();
  /// Destructor.
  virtual ~NodeCreationProxy();
  
  /// Sets the X3DComponentCreator.
  void setComponentCreator(X3DComponentCreator *component);
  /// Creates the SFNode from its name.
  SFNode createFromName(const SFString &name) const;
  /// Empties the list of component.
  void reset();
  
  /// Gets the name of the component of the type name.
  SFString getComponentNameOf(const SFString &name) const;

  friend NodeCreationProxy *joinNodeCreationProxy(NodeCreationProxy *N0, NodeCreationProxy *N1);
  
private:  
  /// Adds a type.
  virtual void addType(const Type *type);
  ///Gets the creation function related with name.
  CreationFunction *getCreationFunctionOf(const SFString &name) const;
  
  //Dictionary of all the creation functions used.
  CreationDict creationDict_;
  std::list<X3DComponentCreator *> componentList_;
};

}

#endif
