/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            Type.h                                          //
////////////////////////////////////////////////////////////////////////////////

#ifndef TYPE_H
#define TYPE_H

#include "X3DTypes.h"

namespace X3DTK {

class Component;
class X3DNodeProxy;

/// Class defining an artificial data structure storing the derivation tree. It is
/// based upon singleton for tag. It means that it exists only one instance of Type 
/// of tag name "Cylinder". 

class Type
{
  friend class NodeCreationProxy;
  friend class NodeWalkingProxy;
  friend class NodeVisitingProxy;
  friend class X3DComponentCreator;
  friend class X3DComponentWalker;
  friend class X3DComponentVisitor;
  friend class X3DAbstractNode;

public:
  /// Gets the Type of name name.
  static Type *getTypeOfName(const SFString &name);
  /// Gets the Type of id id.
  static Type *getTypeOfId(int id);
  /// Recomputes the id and changes all the nodeProxy array.(Not yet implemented)
  static void recomputeIds();
  
  /// Returns the name of the node
  inline SFString getName() const {return _name;};
  /// Return the id of the node.
  inline int getId() const {return _id;};
  /// Gets the parent.
  inline Type *getParent() const {return _parent;};
  /// Gets the list of the children.
  inline MFType getChildList() const {return _childList;};
  /// Gets the component.
  inline Component *getComponent() const {return _component;};
  /// Prints the current derivation tree.
  static void printDerivationTree();

  static int erc;
    
private:
  /// Constructor.
  Type(const SFString &name);
  ~Type();

  /// Sets the parent of the derivation hierarchy.
  void setParent(const Type *parent);
  /// Adds one instance of the type.
  void addOneReference();
  /// Removes one instance of the type.
  static void removeOneReference(Type *type);
  
  /// Defines a new Tag name. Only X3DNode is able to call this method.
  static void defineTagName(Type **type, const SFString &name, const SFString &component);
  /// Adds a new proxy. Only X3DNodeProxy is able to call this method.
  static void addNodeProxy(X3DNodeProxy *proxy);
  /// Removes proxy from the list. Only X3DNodeProxy is able to call this method.
  static void removeNodeProxy(X3DNodeProxy *proxy);

  /// Specifies to Type that the next new Nodes are necessary to get their name and 
  /// will not be used then.
  static void beginFunctionDefinition();
  /// Specifies to Type that the function definition is terminated.
  static void endFunctionDefinition();

  
  /// attributes
  SFString _name;
  int _id;
  int _refCount;
  Type *_parent;
  MFType _childList;
  Component *_component;
  
  /// static attributes
  static int count;
  static MFType _typeList;  
  static MFNodeProxy _proxyList;
  static bool _functionDefinition;
 
  void printDerivationTree(SFString tab) const;
};

}

#endif
