/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            NameDict.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef NAMEDICT_H
#define NAMEDICT_H

#include "X3DTypes.h"

namespace X3DTK {

/// Class providing a dictionary of list of names.

class NameDict
{
public:
  /// Constructor.
  NameDict();
  /// Destructor.
  ~NameDict();
  
  /// Inserts the couple (name, N).
  void insert(const SFString &name, const SFNode &N);
  /// Removes the couple of which the node is N.
  void remove(SFNode N);
  /// Sets the set of DEF non ambiguous, it means that it renames the common DEF. 
  /// It creates new DEF when there are more than one node referenced by the same DEF.
  /// The new name is "DEF"_i i>=0.
  void makeDEFNonAmbiguous();
  
  /// Gets the node corresponding to the name name. If the list contains more than 
  /// One element, the last element inserted is returned.
  SFNode getNodeOfName(const SFString &name) const;
  /// Gets the name corresponding to the node N.
  SFString getNameOfNode(const SFNode &N) const;
  
  ///Prints the content of the dictionary.
  void printDict() const;
  
private:
  X3DDict _nameDict; 
}; 

}
 
#endif
