#include "X3DAbstractNode.h"
#include "Type.h"
#include "Scene.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

NameDict X3DAbstractNode::DEFDict = NameDict();

X3DAbstractNode::X3DAbstractNode()
: _type(0)
{    
  defineTagName("X3DAbstractNode", "Root");
}

SFNode X3DAbstractNode::clone() const
{
  // Returns 0 because it is an abstract node.
  return 0;
}

X3DAbstractNode::X3DAbstractNode(const X3DAbstractNode &N)
: _sceneList(N._sceneList), _type(N._type), _parentList(N._parentList)
{
  defineTagName("X3DAbstractNode", "Root");
}

X3DAbstractNode::~X3DAbstractNode()
{
  //this method is recursive and removes one instance for all the parents of
  //the type.
  Type::removeOneReference(_type);
  X3DAbstractNode::DEFDict.remove(this);
  /// Removes the instance for all the parents
  for (MFNode::iterator it = _parentList.begin(); it != _parentList.end(); ++it)
    (*it)->removeChild(this);
}

void X3DAbstractNode::addParent(const SFNode &N)
{
  if (N != 0)
    _parentList.push_back(N);
}

bool X3DAbstractNode::removeParent(const SFNode &N)
{
  if (N != 0)
  {
    MFNode::iterator res = find(_parentList.begin(), _parentList.end(), N);
    if (res != _parentList.end())
    {
      _parentList.erase(res);
      return true;
    }
  }
  return false;
}

void X3DAbstractNode::loadAttributes(const X3DFileElement *element)
{
}

SFString X3DAbstractNode::writeAttributes() const
{
  return "";
}

bool X3DAbstractNode::addChild(const SFNode &N)
{
  return false;
}

bool X3DAbstractNode::setChild(const SFNode &N)
{
  return false;
}

bool X3DAbstractNode::removeChild(const SFNode &N)
{
  return false;
}

void X3DAbstractNode::removeScenes(SFNode child, const MFScene &sceneList)
{
  //union of the sets.
  if ((child != 0) && (sceneList.size() != 0))
  {
    for (MFScene::iterator it = sceneList.begin(); it != sceneList.end(); ++it)
      child->_sceneList.erase(*it);
      
    child->removeScenesToChildren(sceneList);
  }  
}

void X3DAbstractNode::removeScenesToChildren(const MFScene &sceneList)
{
}

void X3DAbstractNode::addScenes(SFNode child, const MFScene &sceneList)
{
  //union of the sets.
  if ((child != 0) && (sceneList.size() != 0))
  {
    child->_sceneList.insert<MFScene::iterator>(sceneList.begin(), sceneList.end());
    child->addScenesToChildren(sceneList);
  }  
}

void X3DAbstractNode::addScenesToChildren(const MFScene &sceneList)
{
}

void X3DAbstractNode::removeLink(SFNode father, SFNode child)
{
  if (child != 0)
  {
    child->removeParent(father);
    removeScenes(child, father->_sceneList);
  }  
}

void X3DAbstractNode::addLink(SFNode father, SFNode child)
{
  if (child != 0)
  {
    child->addParent(father);
    addScenes(child, father->_sceneList);
  }  
}
