/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            X3DComponentWalker.h                            //
////////////////////////////////////////////////////////////////////////////////

#ifndef X3DCOMPONENTWALKER_H
#define X3DCOMPONENTWALKER_H

#include "X3DComponent.h"
#include "WalkingFunction.h"
#include "NodeCreationProxy.h"
#include "X3DAbstractNode.h"

namespace X3DTK {

class NodeWalkingProxy;
class NodeVisitingProxy;

/// Abstract class defining the concept of component walker.

class X3DComponentWalker : public X3DComponent
{
public:
  /// Constructor.
  X3DComponentWalker();
  /// Destructor.
  virtual ~X3DComponentWalker() = 0;
  
  /// Sets the NodeVisitingProxy
  void setNodeWalkingProxy(const NodeWalkingProxy *nodeWalkingProxy);
  /// Sets the NodeVisitingProxy
  void setNodeVisitingProxy(const NodeVisitingProxy *nodeVisitingProxy);
  
  /// Sets the function active.
  template<class T> 
  void setActiveFunction()
  {
    T *t = new T();
    //iterating the list
    SFString name = t->getName();
    delete t;
    //checking if the node belongs to the component.
/*    if ((nodeCreationProxy_ != 0) && (nodeCreationProxy_->getComponentNameOf(name) != name_))
    {
      std::cerr << "warning: " << name << " doesn't belong to the " << name_ << " component!" << std::endl;
      return;
    }*/
    WalkingDict::const_iterator it = walkingDict_.find(name);
    if (it != walkingDict_.end())
      (*it).second->setActive();
      
  };
  /// Sets the function inactive.
  template<class T> 
  void setInactiveFunction()
  {
    T *t = new T();
    //iterating the list
    SFString name = t->getName();
    delete t;
    //checking if the node belongs to the component.
/*    if ((nodeCreationProxy_ != 0) && (nodeCreationProxy_->getComponentNameOf(name) != name_))
    {
      std::cerr << "warning: " << name << " doesn't belong to the " << name_ << " component!" << std::endl;
      return;
    }*/
    if (name != "X3DNode")
    {
      WalkingDict::const_iterator it = walkingDict_.find(name);
      if (it != walkingDict_.end())
        (*it).second->setInactive();
    } 
  };
  /// Sets all the functions active.
  void setActiveAllFunctions();
  /// Sets all the functions inactive.  
  void setInactiveAllFunctions();
  
  /// Gets the walking function of name.
  WalkingFunction *getWalkingFunctionOf(const SFString &name) const;
  
protected:  
  /// Reference to the NodeWalkingProxy.
  NodeWalkingProxy *nodeWalkingProxy;
  /// Reference to the NodeVisitingProxy.
  NodeVisitingProxy *nodeVisitingProxy;
  /// Defines a new creation function.
  template<class C, class T> 
  void defineNewWalkingFunction(void (C::*ptrF)(T *) const)
  {
    Type::beginFunctionDefinition();
    
    WalkingFunction *WF = new WalkingFunction(reinterpret_cast<ptrToWalkingFunction>(ptrF), this);
    //finding if an occurence of T
    T *t = new T();
    SFString name = t->getTypeName();
    Component *TComponent = t->getType()->getComponent();
    //checking if the node belongs to the component.
    if (TComponent != component)
    {
      std::cerr << "warning: defineNewWalkingFunction for " << name << " which doesn't belong to the " << component->getName() << " component," << std::endl;
      std::cerr << "         but to the " << t->getType()->getComponent()->getName() << " component!" << std::endl;
    }         
    else
      walkingDict_.insert(std::pair<SFString, WalkingFunction *>(name, WF));
    
    delete t;       
    Type::endFunctionDefinition();
  };
  
private:
  WalkingDict walkingDict_;
};

}

#endif
