/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            GLIndexedLineSet.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLINDEXEDLINESET_H
#define GLINDEXEDLINESET_H

#include "X3DGLGeometryNode.h"

#include <vector>

namespace X3DTK {

class IndexedLineSet;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// IndexedLineSet.

class GLIndexedLineSet : public X3DGLGeometryNode
{
public:
  /// Constructor.
  GLIndexedLineSet();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~GLIndexedLineSet();

  /// Gets the lineWidth.
  inline SFFloat &getLineWidth() const {return (SFFloat &)_lineWidth;};
  /// Gets the colorPerVertex.
  inline SFBool &getColorPerVertex() const {return (SFBool &)_colorPerVertex;};
  /// gets the color.
  inline SFBool &getColor() const {return (SFBool &)_color;};
  /// Gets the pointArray.
  inline MFVec3f &getPointArray() const {return (MFVec3f &)_pointArray;};
  /// Gets the lineArrayArray.
  inline std::vector<std::vector<C4UB_V3F> *> &getLineArrayArray() const {return (std::vector<std::vector<C4UB_V3F> *> &)_lineArrayArray;};
  /// Gets the lineIndexArray.
  inline std::vector<MFInt32 *> &getLineIndexArray() const {return (std::vector<MFInt32 *> &)_lineIndexArray;};
  /// Gets the pointLine Array.
  inline std::vector<C4UB_V3F> &getPointLineArray() const {return (std::vector<C4UB_V3F> &)_pointLineArray;};

protected:
  /// Copy constructor.      
  GLIndexedLineSet(const GLIndexedLineSet &I);
  
private:
  //GL attributes
  SFFloat _lineWidth;
  SFBool _colorPerVertex;
  SFBool _color;

  MFVec3f _pointArray;
  std::vector<std::vector<C4UB_V3F> *> _lineArrayArray;
  std::vector<MFInt32 *> _lineIndexArray;
  std::vector<C4UB_V3F> _pointLineArray;
};

}

#endif
