/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            GLTriangleStripSet.h                            //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLTRIANGLESTRIPSET_H
#define GLTRIANGLESTRIPSET_H

#include "X3DGLComposedGeometryNode.h"

#include <vector>

namespace X3DTK {

class TriangleStripSet;

/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// TriangleStripSet.

class GLTriangleStripSet : public X3DGLComposedGeometryNode
{
public:
  /// Constructor.
  GLTriangleStripSet();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~GLTriangleStripSet();
  
  /// Gets the ccw.
  inline SFBool &getCcw() const {return (SFBool &)_ccw;};
  /// Gets the solid.
  inline SFBool &getSolid() const {return (SFBool &)_solid;};
  /// Gets the verticesDuplicated.
  inline SFBool &getVerticesDuplicated() const {return (SFBool &)_verticesDuplicated;};
  /// Gets the color.
  inline SFBool &getColor() const {return (SFBool &)_color;};
  /// Gets the texCoord.
  inline SFBool &getTexCoord() const {return (SFBool &)_texCoord;};

  /// Gets the _N3F_V3FvertexFaceArray.
  inline std::vector<std::vector<N3F_V3F> *> &getN3F_V3FvertexStripArrayArray() const {return (std::vector<std::vector<N3F_V3F> *> &)_N3F_V3FvertexStripArrayArray;};
  /// Gets the _C4F_N3F_V3FvertexFaceArray.
  inline std::vector<std::vector<C4F_N3F_V3F> *> &getC4F_N3F_V3FvertexStripArrayArray() const {return (std::vector<std::vector<C4F_N3F_V3F> *> &)_C4F_N3F_V3FvertexStripArrayArray;};

protected:
  /// Copy constructor.
  GLTriangleStripSet(const GLTriangleStripSet &T);
    
private:
  //GL attributes
  SFBool _ccw;
  SFBool _solid;
  SFBool _verticesDuplicated;
  SFBool _color;
  SFBool _texCoord;
  
  std::vector<std::vector<C4F_N3F_V3F> *> _C4F_N3F_V3FvertexStripArrayArray;  
  std::vector<std::vector<N3F_V3F> *> _N3F_V3FvertexStripArrayArray;  
};

}

#endif
