/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            GLAppearance.h                                  //
////////////////////////////////////////////////////////////////////////////////

#ifndef GLAPPEARANCE_H
#define GLAPPEARANCE_H

#include "X3DGLAppearanceNode.h"

namespace X3DTK {

class GLMaterial;
class X3DGLTextureNode;
class X3DGLTextureTransformNode;
class GLLineProperties;
class GLFillProperties;


/// Class providing a implementation of the X3DGLNode corresponding to the X3DNode 
/// Appearance.

class GLAppearance : public X3DGLAppearanceNode
{
public:
  /// Constructor.
  GLAppearance();
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~GLAppearance();
  
  /// Sets the Material attribute.
  void setMaterial(const GLMaterial *material);
  /// Sets the Texture attribute.
  void setTexture(const X3DGLTextureNode *texture);
  /// Sets the TextureTransform attribute.
  void setTextureTransform(const X3DGLTextureTransformNode *textureTransform);
  /// Sets the LineProperties attribute.
  void setLineProperties(const GLLineProperties *lineProperties);
  /// Sets the FillProperties attribute.
  void setFillProperties(const GLFillProperties *fillProperties);
  
  /// Gets the Material attribute.
  inline SFNode getMaterial() {return _material;};
  /// Gets the Texture attribute.
  inline SFNode getTexture() {return _texture;};
  /// Gets the TextureTransform attribute.
  inline SFNode getTextureTransform() {return _textureTransform;};
  /// Gets the LineProperties attribute.
  inline SFNode getLineProperties() {return _lineProperties;};
  /// Gets the FillProperties attribute.
  inline SFNode getFillProperties() {return _fillProperties;};
   
  /// Adds the child N. Returns FALSE if the child already exists or N is not a 
  /// valid child.   
  virtual bool addChild(const SFNode &N);
  /// Sets the child N. Returns FALSE if N is not a 
  /// valid child. If the child already exists then N becomes the new child.
  virtual bool setChild(const SFNode &N);
  /// Removes the child if exists. Returns TRUE if succeeds.
  virtual bool removeChild(const SFNode &N); 

protected:  
  /// Copy constructor.	     
  GLAppearance(const GLAppearance &A);
  
private:
  SFNode _material;
  SFNode _texture;
  SFNode _textureTransform;
  SFNode _lineProperties;
  SFNode _fillProperties;
};

}

#endif
