#include "X3DGLShapeNode.h"
#include "X3DGLGeometryNode.h"
#include "X3DGLAppearanceNode.h"

using namespace X3DTK;

X3DGLShapeNode::X3DGLShapeNode()
: X3DGLChildNode(), _geometry(0), _appearance(0)
{
  defineTagName("X3DGLShapeNode", "GLShape");
}

X3DGLShapeNode::X3DGLShapeNode(const X3DGLShapeNode &N)
: X3DGLChildNode(N), _geometry(N._geometry), _appearance(N._appearance)
{
  defineTagName("X3DGLShapeNode", "GLShape");
}

SFNode X3DGLShapeNode::clone() const
{
  return 0;
}

X3DGLShapeNode::~X3DGLShapeNode()
{
}

void X3DGLShapeNode::setGeometry(const X3DGLGeometryNode *geometry)
{
  _geometry = (X3DGLGeometryNode *)geometry;
}

void X3DGLShapeNode::setAppearance(const X3DGLAppearanceNode *appearance)
{
  _appearance = (X3DGLAppearanceNode *)appearance;
}

bool X3DGLShapeNode::addChild(const SFNode &N)
{
  if (dynamic_cast<X3DGLGeometryNode *>(N) != 0)
  {
    if (_geometry != 0)
      return false;
      
    _geometry = N;
    return true;
  }
  
  if (dynamic_cast<X3DGLAppearanceNode *>(N) != 0)
  {
    if (_appearance != 0)
      return false;
      
    _appearance = N;
    return true;    
  }
  
  return false;
}

bool X3DGLShapeNode::setChild(const SFNode &N)
{
  if (dynamic_cast<X3DGLGeometryNode *>(N) != 0)
  {
    _geometry = N;
    return true;
  }
  
  if (dynamic_cast<X3DGLAppearanceNode *>(N) != 0)
  {
    _appearance = N;
    return true;    
  }
  
  return false;
}

bool X3DGLShapeNode::removeChild(const SFNode &N)
{
  if (_geometry == N)
  {
    _geometry = 0;
    return true;
  } 
   
  if (_appearance == N)
  {
    _appearance = 0;
    return true;
  } 
  
  return false;
}

void X3DGLShapeNode::removeScenesToChildren(const MFScene &sceneList)
{
  removeScenes(_geometry, sceneList);
  removeScenes(_appearance, sceneList);
}

void X3DGLShapeNode::addScenesToChildren(const MFScene &sceneList)
{
  addScenes(_geometry, sceneList);
  addScenes(_appearance, sceneList);
}
