/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            Cone.h                                          //
////////////////////////////////////////////////////////////////////////////////

#ifndef CONE_H
#define CONE_H

#include "X3DTypes.h"
#include "X3DGeometry3DNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/geometry3D.html">
/// Geometry3D component</a> defining a Cone
/// (<a href="../../doc/X3D_Specs/Part01/components/geometry3D.html#Cone">
/// X3D documentation</a>).

class Cone : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Cone();
  /// Constructs a Cone from its attributes.
  Cone(SFFloat bottomRadius, 
       SFFloat height, 
       SFBool side, 
       SFBool bottom);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Cone();
  
  /// Sets the BottomRadius attribute.
  void setBottomRadius(SFFloat bottomRadius);
  /// Sets the Height attribute.
  void setHeight(SFFloat height);
  /// Sets the Side attribute.
  void setSide(SFBool side);
  /// Sets the Bottom attribute.
  void setBottom(SFBool bottom);
  
  /// Gets the BottomRadius attribute.
  inline SFFloat getBottomRadius() const {return bottomRadius_;};
  /// Gets the Height attribute.
  inline SFFloat getHeight() const {return height_;};
  /// Gets the Side attribute.
  inline SFBool getSide() const {return side_;};
  /// Gets the Bottom attribute.
  inline SFBool getBottom() const {return bottom_;};

  /// Loads the attributes from a X3DFileElement e.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.	   
  Cone(const Cone &C);  
  
private:
  SFFloat bottomRadius_;
  SFFloat height_;
  SFBool side_;
  SFBool bottom_;
};

}

#endif
