#include "Cylinder.h"

using namespace X3DTK;

Cylinder::Cylinder()
: X3DGeometry3DNode(), bottom_(true), radius_(1.0f), height_(2.0f), side_(true), top_(true)  
{
  defineTagName("Cylinder", "Geometry3D");
}

Cylinder::Cylinder(SFBool bottom, SFFloat radius, SFFloat height, SFBool side, SFBool top)
: X3DGeometry3DNode(), bottom_(bottom), radius_(radius), height_(height), side_(side), top_(top)
{
  defineTagName("Cylinder", "Geometry3D");
}

Cylinder::Cylinder(const Cylinder &C)
: X3DGeometry3DNode(C), bottom_(C.bottom_), radius_(C.radius_), height_(C.height_), side_(C.side_), top_(C.top_)
{
  defineTagName("Cylinder", "Geometry3D");
}

SFNode Cylinder::clone() const
{
  return new Cylinder(*this);
}

Cylinder::~Cylinder()
{
}

void Cylinder::setBottom(SFBool bottom)
{
  bottom_ = bottom;
}

void Cylinder::setRadius(SFFloat radius)
{
  radius_ = radius;
}

void Cylinder::setHeight(SFFloat height)
{
  height_ = height;
}

void Cylinder::setSide(SFBool side)
{
  side_ = side;
}

void Cylinder::setTop(SFBool top)
{
  top_ = top;
}

void Cylinder::loadAttributes(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("bottom");
  if (index != -1)
    bottom_ = (element->getAttribute(index).upper() == "TRUE");
  
  index = element->getIndexAttribute("radius");
  if (index != -1)
    radius_ = element->getAttribute(index).toFloat(); 

  index = element->getIndexAttribute("height");
  if (index != -1)
    height_ = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("side");
  if (index != -1)
    side_ = (element->getAttribute(index).upper() == "TRUE"); 
  
  index = element->getIndexAttribute("top");
  if (index != -1)
    top_ = (element->getAttribute(index).upper() == "TRUE"); 
}

SFString Cylinder::writeAttributes() const
{
  SFString attr;
  if (!bottom_)
    attr += " bottom=\"FALSE\"";
  if (radius_ != 1.0f)  
    attr += " radius=\"" + toSFString(radius_) + "\"";
  if (height_ != 2.0f)  
    attr += " height=\"" + toSFString(height_) + "\"";
  if (!side_)  
    attr += " side=\"FALSE\"";
  if (!top_)    
    attr += " top=\"FALSE\"";
      
  return attr;
}
