/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            Cylinder.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef CYLINDER_H
#define CYLINDER_H

#include "X3DTypes.h"
#include "X3DGeometry3DNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/geometry3D.html">
/// Geometry3D component</a> defining a Box
/// (<a href="../../doc/X3D_Specs/Part01/components/geometry3D.html#Cylinder">
/// X3D documentation</a>).

class Cylinder : public X3DGeometry3DNode
{
public:
  /// Constructor.
  Cylinder();
  /// Constructs a Cylinder from its attributes.
  Cylinder(SFBool bottom, 
           SFFloat radius, 
	   SFFloat height, 
	   SFBool side, 
	   SFBool top);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Cylinder();
  
  /// Sets the bottom attribute.
  void setBottom(SFBool bottom);
  /// Sets the radius attribute.
  void setRadius(SFFloat radius);
  /// Sets the height attribute.
  void setHeight(SFFloat height);
  /// Sets the side attribute.
  void setSide(SFBool side);
  /// Sets the top attribute.
  void setTop(SFBool top);
  
  /// Gets the bottom attribute.
  inline SFBool getBottom() const {return bottom_;};
  /// Gets the radius attribute.
  inline SFFloat getRadius() const {return radius_;};
  /// Gets the height attribute.
  inline SFFloat getHeight() const {return height_;};
  /// Gets the side attribute.
  inline SFBool getSide() const {return side_;};
  /// Gets the top attribute.
  inline SFBool getTop() const {return top_;};
  
  /// Loads the attributes from a X3DFileElement e.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.	   
  Cylinder(const Cylinder &C);

private:
  SFBool bottom_;
  SFFloat radius_;
  SFFloat height_;
  SFBool side_;
  SFBool top_;
};

}

#endif
