#include "IndexedFaceSet.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

IndexedFaceSet::IndexedFaceSet()
: X3DComposedGeometryNode(), convex_(true), creaseAngle_(0.0f)
{
  defineTagName("IndexedFaceSet", "Geometry3D");
}

IndexedFaceSet::IndexedFaceSet(const X3DColorNode *color, const X3DCoordinateNode *coord, const X3DNormalNode *normal, const X3DTextureCoordinateNode *texCoord, SFBool ccw, const MFInt32 &colorIndex, SFBool colorPerVertex, SFBool convex, const MFInt32 &coordIndex, SFFloat creaseAngle, const MFInt32 &normalIndex, SFBool normalPerVertex, SFBool solid, const MFInt32 &texCoordIndex) 
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
colorIndex_(colorIndex), convex_(convex), coordIndex_(coordIndex), creaseAngle_(creaseAngle), normalIndex_(normalIndex), texCoordIndex_(texCoordIndex)
{
  defineTagName("IndexedFaceSet", "Geometry3D");
}
           
IndexedFaceSet::IndexedFaceSet(const IndexedFaceSet &I)
: X3DComposedGeometryNode(I),
colorIndex_(I.colorIndex_), convex_(I.convex_), coordIndex_(I.coordIndex_), creaseAngle_(I.creaseAngle_), normalIndex_(I.normalIndex_), texCoordIndex_(I.texCoordIndex_)
{
  defineTagName("IndexedFaceSet", "Geometry3D");
}

SFNode IndexedFaceSet::clone() const
{
  return new IndexedFaceSet(*this);
}

IndexedFaceSet::~IndexedFaceSet()
{
}
  
void IndexedFaceSet::setColorIndex(const MFInt32 &colorIndex)
{
  colorIndex_ = colorIndex;
}

void IndexedFaceSet::setConvex(SFBool convex)
{
  convex_ = convex;
}

void IndexedFaceSet::setCoordIndex(const MFInt32 &coordIndex)
{
  coordIndex_ = coordIndex;
}

void IndexedFaceSet::setCreaseAngle(SFFloat creaseAngle)
{
  creaseAngle_ = creaseAngle;
}

void IndexedFaceSet::setNormalIndex(const MFInt32 &normalIndex)
{
  normalIndex_ = normalIndex;
}

void IndexedFaceSet::setTexCoordIndex(const MFInt32 &texCoordIndex)  
{
  texCoordIndex_ = texCoordIndex;
}

void IndexedFaceSet::loadAttributes(const X3DFileElement *element)
{
  X3DComposedGeometryNode::loadAttributes(element);
 
  int index;
  index = element->getIndexAttribute("colorIndex");
  if (index != -1)
    colorIndex_ = MFInt32(element->getAttribute(index));
  
  index = element->getIndexAttribute("convex");  
  if (index != -1)
    convex_ = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("coordIndex");
  if (index != -1)
    coordIndex_ = MFInt32(element->getAttribute(index));

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    creaseAngle_ = element->getAttribute(index).toFloat(); 
  
  index = element->getIndexAttribute("normalIndex");
  if (index != -1)
    normalIndex_ = MFInt32(element->getAttribute(index));

  index = element->getIndexAttribute("texCoordIndex");
  if (index != -1)
    texCoordIndex_ = MFInt32(element->getAttribute(index));
}

SFString IndexedFaceSet::writeAttributes() const
{
  SFString attr;
  attr = X3DComposedGeometryNode::writeAttributes();
  
  if (!colorIndex_.empty()) 
    attr += " colorIndex=\"" + toSFString(colorIndex_) + "\"";
  if (!convex_)  
    attr += " convex=\"FALSE\"";
  if (!coordIndex_.empty())  
    attr += " coordIndex=\"" + toSFString(coordIndex_) + "\"";
  if (creaseAngle_ != 0.0f)
    attr += " creaseAngle=\"" + toSFString(creaseAngle_) + "\"";
  if (!normalIndex_.empty())  
    attr += " normalIndex=\"" + toSFString(normalIndex_) + "\"";
  if (!texCoordIndex_.empty())  
    attr += " texCoordIndex=\"" + toSFString(texCoordIndex_) + "\"";
  
  return attr;
}
