/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            Transform.h                                     //
////////////////////////////////////////////////////////////////////////////////

#ifndef TRANSFORM_H
#define TRANSFORM_H

#include "X3DTypes.h"
#include "X3DGroupingNode.h"
#include "X3DBoundedObject.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/group.html">
/// Grouping component</a> defining a Transform
/// (<a href="../../doc/X3D_Specs/Part01/components/grouping.html#Transform">
/// X3D documentation</a>).

class Transform : public X3DGroupingNode, public X3DBoundedObject
{
public:  
  /// Constructor.
  Transform();
  /// Constructs a Transform from its attributes.
  Transform(const SFVec3f &center, 
	    const SFRotation &rotation, 
	    const SFVec3f &scale, 
	    const SFRotation &scaleOrientation, 
	    const SFVec3f &translation, 
	    const SFVec3f &bboxCenter, 
	    const SFVec3f &bboxSize);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Transform();

  /// Sets the center of the transformation.
  void setCenter(const SFVec3f &center);
  /// Sets the rotation of the transformation.
  void setRotation(const SFRotation &rotation);
  /// Sets the scale of the transformation.
  void setScale(const SFVec3f &scale);
  /// Sets the scale orientation of the transformation.
  void setScaleOrientation(const SFRotation &scaleOrientation);
  /// Sets the translation of the transformation.
  void setTranslation(const SFVec3f &translation);

  /// Gets the center of the transformation.
  inline SFVec3f getCenter() const {return center_;};
  /// Gets the rotation of the transformation.
  inline SFRotation getRotation() const {return rotation_;};
  /// Gets the scale of the transformation.
  inline SFVec3f getScale() const {return scale_;};
  /// Gets the scale orientation of the transformation.
  inline SFRotation getScaleOrientation() const {return scaleOrientation_;};
  /// Gets the translation of the transformation.
  inline SFVec3f getTranslation() const {return translation_;};

  /// Loads the attributes from a X3DFileElement *.
  void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor.
  Transform(const Transform &T);

private:
  SFVec3f center_;
  SFRotation rotation_;
  SFVec3f scale_;
  SFRotation scaleOrientation_;
  SFVec3f translation_;
};

}

#endif
