/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            Normal.h                                        //
////////////////////////////////////////////////////////////////////////////////

#ifndef NORMAL_H
#define NORMAL_H

#include "X3DTypes.h"
#include "X3DNormalNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining a Normal
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#Normal">
/// X3D documentation</a>).

class Normal : public X3DNormalNode
{
public:
  /// Constructor.
  Normal();
  /// Constructs a Normal from its attributes.
  Normal(const MFVec3f &vector);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Normal();
  
  /// Sets the Vector attribute.
  void setVector(const MFVec3f &vector);
  
  /// Gets the Vector attribute.
  inline MFVec3f &getVector() const {return (MFVec3f &)vector_;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;
  
protected: 
  /// Copy constructor.
  Normal(const Normal &N);
private:
  MFVec3f vector_;  
};

}

#endif
