#include "PointSet.h"
#include "RenderingNodes.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

PointSet::PointSet()
: X3DGeometryNode(), color_(0), coord_(0)
{
  defineTagName("PointSet", "Rendering");
}

PointSet::PointSet(const X3DColorNode *color, const X3DCoordinateNode *coord)
: X3DGeometryNode(), color_((SFNode)color), coord_((SFNode)coord)
{
  defineTagName("PointSet", "Rendering");
}
 
           
PointSet::PointSet(const PointSet &P)
: X3DGeometryNode(P), color_(P.color_), coord_(P.coord_)
{
  defineTagName("PointSet", "Rendering");
}

SFNode PointSet::clone() const
{
  return new PointSet(*this);
}

PointSet::~PointSet()
{
}

//modifiers
void PointSet::setColor(const X3DColorNode *color)
{
  removeLink(this, color_);
  color_ = (SFNode)color;
  removeLink(this, color_);
}

void PointSet::setCoord(const X3DCoordinateNode *coord)
{
  removeLink(this, coord_);
  coord_ = (SFNode)coord;
  removeLink(this, coord_);
}

bool PointSet::addChild(const SFNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    if (color_ != 0)
      return false;
      
    setColor(static_cast<X3DColorNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    if (coord_ != 0)
      return false;
      
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  cerr << "PointSet::addChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;
}

bool PointSet::setChild(const SFNode &N)
{
  if (dynamic_cast<X3DColorNode *>(N) != 0)
  {
    setColor(static_cast<X3DColorNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DCoordinateNode *>(N) != 0)
  {
    setCoord(static_cast<X3DCoordinateNode *>(N));
    return true;    
  }
  
  cerr << "PointSet::setChild : a node of type " << N->getTypeName() << " cannot be a child!" << endl;
  return false;
}

bool PointSet::removeChild(const SFNode &N)
{
  if (color_ == N)
  {
    setColor(0);
    return true;
  } 
   
  if (coord_ == N)
  {
    setCoord(0);
    return true;
  } 
  
  return false;
}

void PointSet::removeScenesToChildren(const MFScene &sceneList)
{
  removeScenes(color_, sceneList);
  removeScenes(coord_, sceneList);
}

void PointSet::addScenesToChildren(const MFScene &sceneList)
{
  addScenes(color_, sceneList);
  addScenes(coord_, sceneList);
}
