#include "TriangleStripSet.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

TriangleStripSet::TriangleStripSet()
: X3DComposedGeometryNode(), convex_(true), creaseAngle_(0.0f)
{
  defineTagName("TriangleStripSet", "Rendering");
}

TriangleStripSet::TriangleStripSet(const X3DColorNode *color, const X3DCoordinateNode *coord, const X3DNormalNode *normal, const X3DTextureCoordinateNode *texCoord, SFBool ccw, SFBool colorPerVertex, SFBool convex, SFFloat creaseAngle, SFBool normalPerVertex, SFBool solid, const MFInt32 &stripCount)
: X3DComposedGeometryNode(color, coord, normal, texCoord, ccw, colorPerVertex, normalPerVertex, solid),
convex_(convex), creaseAngle_(creaseAngle), stripCount_(stripCount)
{
  defineTagName("TriangleStripSet", "Rendering");
}

           
TriangleStripSet::TriangleStripSet(const TriangleStripSet &T)
: X3DComposedGeometryNode(T), convex_(T.convex_), creaseAngle_(T.creaseAngle_), stripCount_(T.stripCount_)
{
  defineTagName("TriangleStripSet", "Rendering");
}

SFNode TriangleStripSet::clone() const
{
  return new TriangleStripSet(*this);
}

TriangleStripSet::~TriangleStripSet()
{
}

  
void TriangleStripSet::setConvex(SFBool convex)
{
  convex_ = convex;
}

void TriangleStripSet::setCreaseAngle(SFFloat creaseAngle)
{
  creaseAngle_ = creaseAngle;
}

void TriangleStripSet::setStripCount(const MFInt32 &stripCount)
{
  stripCount_  = stripCount;
}

void TriangleStripSet::loadAttributes(const X3DFileElement *element)
{
  X3DComposedGeometryNode::loadAttributes(element);
  
  int index;
  index = element->getIndexAttribute("convex");  
  if (index != -1)
    convex_ = (element->getAttribute(index).upper() == "TRUE");

  index = element->getIndexAttribute("creaseAngle");
  if (index != -1)
    creaseAngle_ = element->getAttribute(index).toFloat(); 
    
  index = element->getIndexAttribute("stripCount");  
  if (index != -1)
    stripCount_ = MFInt32(element->getAttribute(index));  
}

SFString TriangleStripSet::writeAttributes() const
{
  SFString attr;
  attr = X3DComposedGeometryNode::writeAttributes();
  
  if (!convex_)  
    attr += " convex=\"FALSE\"";
  if (creaseAngle_ != 0.0f)
    attr += " creaseAngle=\"" + toSFString(creaseAngle_) + "\"";
  if (!stripCount_.empty())
    attr += " stripCount=\"" + toSFString(stripCount_) + "\"";
    
  return attr;
}
