/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            TriangleStripSet.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef TRIANGLESTRIPSET_H
#define TRIANGLESTRIPSET_H

#include "X3DTypes.h"
#include "X3DComposedGeometryNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/rendering.html">
/// Rendering component</a> defining a TriangleStripSet
/// (<a href="../../doc/X3D_Specs/Part01/components/rendering.html#TriangleStripSet">
/// X3D documentation</a>).

class TriangleStripSet : public X3DComposedGeometryNode
{
public:
  /// Constructor.
  TriangleStripSet();
  /// Constructs a triangleFanSet from its attributes.
  TriangleStripSet(const X3DColorNode *color,
                   const X3DCoordinateNode *coord,
                   const X3DNormalNode *normal,
                   const X3DTextureCoordinateNode *texCoord,
                   SFBool ccw,
                   SFBool colorPerVertex,
                   SFBool convex,
                   SFFloat creaseAngle,
                   SFBool normalPerVertex,
                   SFBool solid,
                   const MFInt32 &stripCount);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  virtual ~TriangleStripSet();
  
  /// Sets the Convex attribute.
  void setConvex(SFBool convex);
  /// Sets the CreaseAngle attribute.
  void setCreaseAngle(SFFloat creaseAngle);
  /// Sets the NormalPerVertex attribute.
  void setStripCount(const MFInt32 &stripCount);
  
  /// Gets the Convex attribute.
  inline SFBool getConvex() const {return convex_;};
  /// Gets the CreaseAngle attribute.
  inline SFFloat getCreaseAngle() const {return creaseAngle_;};
  /// Gets the StripCount attribute.
  inline MFInt32 &getStripCount() const {return (MFInt32 &)stripCount_;};
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;
  
protected:
  /// Copy constructor.
  TriangleStripSet(const TriangleStripSet &T);
  
private:
  SFBool convex_;  
  SFFloat creaseAngle_; 
  MFInt32 stripCount_;
};

}

#endif
