#include "LineProperties.h"

using namespace X3DTK;

LineProperties::LineProperties()
: X3DAppearanceChildNode(), lineStyle_(0), width_(0.0f)
{
  defineTagName("LineProperties", "Shape");
}

LineProperties::LineProperties(SFInt32 lineStyle, SFFloat width)
: X3DAppearanceChildNode(), lineStyle_(lineStyle), width_(width)
{
  defineTagName("LineProperties", "Shape");
}

LineProperties::LineProperties(const LineProperties &L)
: X3DAppearanceChildNode(L), lineStyle_(L.lineStyle_), width_(L.width_)
{
  defineTagName("LineProperties", "Shape");
}

SFNode LineProperties::clone() const
{
  return new LineProperties(*this);
}

LineProperties::~LineProperties()
{
}

void LineProperties::setLineStyle(SFInt32 lineStyle)
{
  lineStyle_ = lineStyle;
}

void LineProperties::setWidth(SFFloat width)
{
  width_ = width;
}

void LineProperties::loadAttributes(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("lineStyle");
  if (index != -1)
   lineStyle_ = element->getAttribute(index).toInt();
   
  index = element->getIndexAttribute("width");
  if (index != -1)
   width_ = element->getAttribute(index).toFloat(); 
}

SFString LineProperties::writeAttributes() const
{
  SFString attr;
  if (lineStyle_ != 0)
    attr += " lineStyle = \"" + toSFString(lineStyle_) + "\"";  
  if (width_ != 0.0f)
    attr += " width = \"" + toSFString(width_) + "\"";
      
  return attr;
}
