#include "Material.h"

using namespace X3DTK;

Material::Material()
: X3DMaterialNode(), ambientIntensity_(0.2f), diffuseColor_(SFColor(0.8f, 0.8f, 0.8f)), emissiveColor_(SFColor(0.0f, 0.0f, 0.0f)), shininess_(0.2f), specularColor_(SFColor(0.0f, 0.0f, 0.0f)), transparency_(0.0f)
{ 
  defineTagName("Material", "Shape");
}

Material::Material(SFFloat ambientIntensity, SFColor diffuseColor, SFColor emissiveColor, SFFloat shininess, SFColor specularColor, SFFloat transparency)
: X3DMaterialNode(), ambientIntensity_(ambientIntensity), diffuseColor_(diffuseColor), emissiveColor_(emissiveColor), shininess_(shininess), specularColor_(specularColor), transparency_(transparency)
{
  defineTagName("Material", "Shape");
}

Material::Material(const Material &M)
: X3DMaterialNode(M), ambientIntensity_(M.ambientIntensity_), diffuseColor_(M.diffuseColor_), emissiveColor_(M.emissiveColor_), shininess_(M.shininess_), specularColor_(M.specularColor_), transparency_(M.transparency_)
{
  defineTagName("Material", "Shape");
}

SFNode Material::clone() const
{
  return new Material(*this);
}

Material::~Material()
{
}

void Material::setAmbientIntensity(SFFloat ambientIntensity)
{
  ambientIntensity_ = ambientIntensity;
}

void Material::setDiffuseColor(SFColor diffuseColor)
{
  diffuseColor_ = diffuseColor;
}

void Material::setEmissiveColor(SFColor emissiveColor)  
{
  emissiveColor_ = emissiveColor;
}

void Material::setShininess(SFFloat shininess)  
{
  shininess_ = shininess;
}

void Material::setSpecularColor(SFColor specularColor)  
{
  specularColor_ = specularColor;  
}

void Material::setTransparency(SFFloat transparency)  
{
  transparency_ = transparency;
}

void Material::loadAttributes(const X3DFileElement *element)
{
  int index;
  index = element->getIndexAttribute("ambientIntensity");
  if (index != -1)
    ambientIntensity_ = element->getAttribute(index).toFloat();

  index = element->getIndexAttribute("diffuseColor");
  if (index != -1)
    diffuseColor_ = SFColor(element->getAttribute(index));

  index = element->getIndexAttribute("emissiveColor");
  if (index != -1)
    emissiveColor_ = SFColor(element->getAttribute(index));

  index = element->getIndexAttribute("shininess");
  if (index != -1)
    shininess_ = element->getAttribute(index).toFloat();

  index = element->getIndexAttribute("specularColor");
  if (index != -1)
    specularColor_ = SFColor(element->getAttribute(index));
}

SFString Material::writeAttributes() const
{
  SFString attr;
  if (ambientIntensity_ != 0.2f)
    attr += " ambientIntensity=\"" + toSFString(ambientIntensity_) + "\"";
  if (diffuseColor_ != SFColor(0.8f, 0.8f, 0.8f))  
    attr += " diffuseColor=\"" + toSFString(diffuseColor_) + "\"";
  if (emissiveColor_ != SFColor(0.0f, 0.0f, 0.0f))  
    attr += " emissiveColor=\"" + toSFString(emissiveColor_) + "\"";
  if (shininess_ != 0.2f)
    attr += " shininess=\"" + toSFString(shininess_) + "\"";
  if (specularColor_ != SFColor(0.0f, 0.0f, 0.0f))
    attr += " specularColor=\"" + toSFString(specularColor_) + "\"";
  if (transparency_ != 0.0f)
    attr += " transparency=\"" + toSFString(transparency_) + "\"";
      
  return attr;
}
