/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            Material.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef MATERIAL_H
#define MATERIAL_H

#include "X3DTypes.h"
#include "X3DMaterialNode.h"

namespace X3DTK {

/// Class of the <a href="../../doc/X3D_Specs/Part01/components/shape.html">Shape 
/// component</a> defining a Material
/// (<a href="../../doc/X3D_Specs/Part01/components/Shape.html#Material">
/// X3D documentation</a>).

class Material : public X3DMaterialNode
{
public:
  /// Constructor.
  Material();
  /// Constructs a Material from its attributes.
  Material(SFFloat ambientIntensity, 
           SFColor diffuseColor, 
	   SFColor emissiveColor, 
	   SFFloat shininess, 
	   SFColor specularColor, 
	   SFFloat transparency);
  /// Clones the node.
  virtual SFNode clone() const;
  /// Destructor.
  ~Material();
  
  /// Sets the AmbientIntensity attribute.
  void setAmbientIntensity(SFFloat ambientIntensity);
  /// Sets the DiffuseColo attribute.
  void setDiffuseColor(SFColor diffuseColor);
  /// Sets the EmissiveColo attribute.
  void setEmissiveColor(SFColor emissiveColor);
  /// Sets the Shininess attribute.
  void setShininess(SFFloat shininess);  
  /// Sets the SpecularColor attribute.
  void setSpecularColor(SFColor specularColor);  
  /// Sets the Transparenc attribute.
  void setTransparency(SFFloat transparency);  
  
  /// Gets the AmbientIntensity attribute.
  inline SFFloat getAmbientIntensity() const {return ambientIntensity_;};       
  /// Gets the DiffuseColor attribute.
  inline SFColor getDiffuseColor() const {return diffuseColor_;};     
  /// Gets the EmissiveColor attribute.
  inline SFColor getEmissiveColor() const {return emissiveColor_; };    
  /// Gets the Shininess attribute.
  inline SFFloat getShininess() const {return shininess_;};        
  /// Gets the SpecularColor attribute.
  inline SFColor getSpecularColor() const {return specularColor_;};    
  /// Gets the Transparency attribute.
  inline SFFloat getTransparency() const {return transparency_;};   
  
  /// Loads the attributes from a X3DFileElement *.
  virtual void loadAttributes(const X3DFileElement *element);
  /// Writes the attributes of the node.
  virtual SFString writeAttributes() const;

protected:
  /// Copy constructor. 
  Material(const Material &M);
    
private:
  SFFloat ambientIntensity_;       
  SFColor diffuseColor_;     
  SFColor emissiveColor_;    
  SFFloat shininess_;        
  SFColor specularColor_;    
  SFFloat transparency_;     
};

}

#endif
