#include "X3DShapeNode.h"
#include "ShapeNodes.h"

#include <iostream>

using namespace X3DTK;
using namespace std;

X3DShapeNode::X3DShapeNode()
: X3DChildNode(), geometry_(0), appearance_(0)
{
  defineTagName("X3DShapeNode", "Shape");
}

X3DShapeNode::X3DShapeNode(const X3DGeometryNode *geometry,  const X3DAppearanceNode *appearance)
: X3DChildNode(), geometry_((SFNode)geometry), appearance_((SFNode)appearance)
{
  defineTagName("X3DShapeNode", "Shape");
}

X3DShapeNode::X3DShapeNode(const X3DShapeNode &N)
: X3DChildNode(N), geometry_(N.geometry_), appearance_(N.appearance_)
{
  defineTagName("X3DShapeNode", "Shape");
}

SFNode X3DShapeNode::clone() const
{
  return 0;
}

X3DShapeNode::~X3DShapeNode()
{
}

void X3DShapeNode::setGeometry(const X3DGeometryNode *geometry)
{
  removeLink(this, geometry_);
  geometry_ = (SFNode)geometry;
  addLink(this, geometry_);
}

void X3DShapeNode::setAppearance(const X3DAppearanceNode *appearance)
{
  removeLink(this, appearance_);
  appearance_ = (SFNode)appearance;
  addLink(this, appearance_);
}

bool X3DShapeNode::addChild(const SFNode &N)
{
  if (dynamic_cast<X3DGeometryNode *>(N) != 0)
  {
    if (geometry_ != 0)
      return false;
      
    setGeometry(static_cast<X3DGeometryNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DAppearanceNode *>(N) != 0)
  {
    if (appearance_ != 0)
      return false;
      
    setAppearance(static_cast<X3DAppearanceNode *>(N));
    return true;    
  }
  
  return false;
}

bool X3DShapeNode::setChild(const SFNode &N)
{
  if (dynamic_cast<X3DGeometryNode *>(N) != 0)
  {
    setGeometry(static_cast<X3DGeometryNode *>(N));
    return true;
  }
  
  if (dynamic_cast<X3DAppearanceNode *>(N) != 0)
  {
    setAppearance(static_cast<X3DAppearanceNode *>(N));
    return true;    
  }
  
  return false;
}

bool X3DShapeNode::removeChild(const SFNode &N)
{
  if (geometry_ == N)
  {
    setGeometry(0);
    return true;
  } 
   
  if (appearance_ == N)
  {
    setAppearance(0);
    return true;
  } 
  
  return false;
}

void X3DShapeNode::removeScenesToChildren(const MFScene &sceneList)
{
  removeScenes(geometry_, sceneList);
  removeScenes(appearance_, sceneList);
}

void X3DShapeNode::addScenesToChildren(const MFScene &sceneList)
{
  addScenes(geometry_, sceneList);
  addScenes(appearance_, sceneList);
}
