/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            SFString.h                                      //
////////////////////////////////////////////////////////////////////////////////

#ifndef SFSTRING_H
#define SFSTRING_H

#include <string>

namespace X3DTK {

/// Class providing a string derived from std::string. Functions are added like the 
/// different conversions.

class SFString : public std::string
{
public:
  /// Default constructor.
  SFString();
  /// Conversion from char *.
  SFString(const char *s);
  /// Conversion from std::string.
  SFString(const std::string &s);
  /// Copy constructor.
  SFString(const SFString &s);
  
  /// Conversion to char *
  operator const char *() const;
  /// Conversion to int.  
  int toInt() const;
  /// Conversion to unsigned int.  
  unsigned int toUint() const;
  /// Conversion to float.  
  float toFloat() const;
  /// Conversion to double.  
  double toDouble() const;
  /// Returns a string that is the string converted to lowercase.
  SFString lower() const;
  /// Returns a string that is the string converted to uppercase.
  SFString upper() const;
  /// Returns the path of the string supposed to be a file.
  SFString getPath() const;
  
  /// Converts a floating number to a SFString.  
  static SFString number(float f);
  /// Converts a double number to a SFString.
  static SFString number(double d);
  /// Converts an integer to a SFString.
  static SFString number(int i); 
  /// Converts an unsigned integer to a SFString.
  static SFString number(unsigned int ui); 
};

}

#endif
