/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            BboxUpdaterGlobalVariables.h                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef BBOXUPDATERGLOBALVARIABLES_H
#define BBOXUPDATERGLOBALVARIABLES_H

#include "GlobalVariables.h"
#include "Bbox.h"

#include <list>

namespace X3DTK {

/// GlobalVariables for the BboxUpdater module.

class BboxUpdaterGlobalVariables : public GlobalVariables
{
public:
  /// Constructor.
  BboxUpdaterGlobalVariables();
  /// Destructor.
  virtual ~BboxUpdaterGlobalVariables();
  
  /// Finishes the traversal.
  virtual void finish();
  
  /// Sets the staticProcessing.
  void setStaticProcessing(bool value);
  /// Adds the Bounding Box BB.
  void addBbox(SFNode N, Bbox *BB);
  /// Sets the current Bbox (for the Geometry nodes of Shape).
  void setShapeBbox(const Bbox &BB);
  /// Adds the bounding Box to the list.
  void addBboxToMergeList(const Bbox &BB);
  
  /// Gets the staticProcessing value.
  inline bool getStaticProcessing() const {return _staticProcessing;};
  /// Gets the current Bbox.
  Bbox getShapeBbox() const {return _shapeBbox;};
  /// Gets the Bbox of N.
  Bbox *getBbox(SFNode N) const;
  /// Merges the Bbox of the list.
  Bbox mergeBbox();
  
private:
  bool _staticProcessing;
  Bbox _shapeBbox;
  std::list<std::pair<SFNode, Bbox *> > _BBList;
  std::list<Bbox> _BBToMergeList;  
};

}

#endif
