#include "GLRendererGLRenderingVisitor.h"
#include "GLIndexedLineSet.h"
#include "GLPointSet.h"
#include "GLTriangleFanSet.h"
#include "GLTriangleSet.h"
#include "GLTriangleStripSet.h"

#include <vector>

using namespace X3DTK;
using namespace std;

GLRendererGLRenderingVisitor::GLRendererGLRenderingVisitor()
: GLRenderingVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLRendererGLRenderingVisitor, GLIndexedLineSet>(&GLRendererGLRenderingVisitor::enterGLIndexedLineSet);
  defineNewEnterFunction<GLRendererGLRenderingVisitor, GLPointSet>(&GLRendererGLRenderingVisitor::enterGLPointSet);
  defineNewEnterFunction<GLRendererGLRenderingVisitor, GLTriangleFanSet>(&GLRendererGLRenderingVisitor::enterGLTriangleFanSet);
  defineNewEnterFunction<GLRendererGLRenderingVisitor, GLTriangleSet>(&GLRendererGLRenderingVisitor::enterGLTriangleSet);
  defineNewEnterFunction<GLRendererGLRenderingVisitor, GLTriangleStripSet>(&GLRendererGLRenderingVisitor::enterGLTriangleStripSet);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLRendererGlobalVariables>();
}

GLRendererGLRenderingVisitor::~GLRendererGLRenderingVisitor()
{
}

void GLRendererGLRenderingVisitor::enterGLIndexedLineSet(GLIndexedLineSet *G) const
{
  glLineWidth(G->getLineWidth());
    
  if (G->getColor())
  {
    glEnable(GL_COLOR_MATERIAL);
    
    if (G->getColorPerVertex())
    {
      for (vector<vector<C4UB_V3F> *>::const_iterator itLineArray = G->getLineArrayArray().begin(); itLineArray != G->getLineArrayArray().end(); ++itLineArray)
      {
        glInterleavedArrays(GL_C4UB_V3F, 0, &(*itLineArray)->front());    
        glDrawArrays(GL_LINE_STRIP, 0, (*itLineArray)->size());
      }
    }
    else
    {
      glInterleavedArrays(GL_C4UB_V3F, 0, &G->getPointLineArray().front());
      glDrawArrays(GL_LINES, 0, G->getPointLineArray().size());      
    }
    glDisable(GL_COLOR_MATERIAL);
  }
  else
  {
    glInterleavedArrays(GL_V3F, 0, &G->getPointArray().front());    
    for (vector<MFInt32 *>::const_iterator itLine = G->getLineIndexArray().begin(); itLine != G->getLineIndexArray().end(); ++itLine)
    {
      glDrawElements(GL_LINE_STRIP, (*itLine)->size(), GL_UNSIGNED_INT, &(*itLine)->front());
    }
  }
}

void GLRendererGLRenderingVisitor::enterGLPointSet(GLPointSet *G) const
{
  glPointSize(4.0f);
  
  if (G->getColor())
  {
    glEnable(GL_COLOR_MATERIAL);
    glInterleavedArrays(GL_C4UB_V3F, 0, &G->getPointSetArray().front());
    glDrawArrays(GL_POINTS, 0, G->getPointSetArray().size());  
    glDisable(GL_COLOR_MATERIAL);
  }
  else
  {
    //drawing the point set
    glInterleavedArrays(GL_V3F, 0, &G->getPointArray().front());
    glDrawArrays(GL_POINTS, 0, G->getPointArray().size());  
  }
}

void GLRendererGLRenderingVisitor::enterGLTriangleFanSet(GLTriangleFanSet *G) const
{ 
  if (G->getSolid())
  {
    glEnable(GL_CULL_FACE);
    if (G->getCcw())
      glFrontFace(GL_CCW);
    else
      glFrontFace(GL_CW);
    glCullFace(GL_BACK);   
  }
  
  if ((G->getColor()) && (G->getTexCoord()))
  {
  }
  
  if ((G->getColor()) && (!G->getTexCoord()))
  {
    glEnable(GL_COLOR_MATERIAL);
    if (!G->getVerticesDuplicated())
    {
      for (vector<vector<C4F_N3F_V3F> *>::const_iterator itFanArray = G->getC4F_N3F_V3FvertexFanArrayArray().begin(); itFanArray != G->getC4F_N3F_V3FvertexFanArrayArray().end(); ++itFanArray)
      {
        glInterleavedArrays(GL_C4F_N3F_V3F, 0, &(*itFanArray)->front());
        glDrawArrays(GL_TRIANGLE_FAN, 0, (*itFanArray)->size());
      }
    }
    else
    {
      for (vector<vector<C4F_N3F_V3F> *>::const_iterator itFanArray = G->getC4F_N3F_V3FvertexFanArrayArray().begin(); itFanArray != G->getC4F_N3F_V3FvertexFanArrayArray().end(); ++itFanArray)
      {
        glInterleavedArrays(GL_C4F_N3F_V3F, 0, &(*itFanArray)->front());
        glDrawArrays(GL_TRIANGLES, 0, (*itFanArray)->size());
      } 
    }
    glDisable(GL_COLOR_MATERIAL);
  }
  
  if ((!G->getColor()) && (G->getTexCoord()))
  {
  }
  
  if ((!G->getColor()) && (!G->getTexCoord()))
  {
    if (!G->getVerticesDuplicated())
    {
      for (vector<vector<N3F_V3F> *>::const_iterator itFanArray = G->getN3F_V3FvertexFanArrayArray().begin(); itFanArray != G->getN3F_V3FvertexFanArrayArray().end(); ++itFanArray)
      {
        glInterleavedArrays(GL_N3F_V3F, 0, &(*itFanArray)->front());
        glDrawArrays(GL_TRIANGLE_FAN, 0, (*itFanArray)->size());
      }
    }
    else
    {
      for (vector<vector<N3F_V3F> *>::const_iterator itFanArray = G->getN3F_V3FvertexFanArrayArray().begin(); itFanArray != G->getN3F_V3FvertexFanArrayArray().end(); ++itFanArray)
      {
        glInterleavedArrays(GL_N3F_V3F, 0, &(*itFanArray)->front());
        glDrawArrays(GL_TRIANGLES, 0, (*itFanArray)->size());
      } 
    }
  }
  
  if (G->getSolid())
    glDisable(GL_CULL_FACE);
}

void GLRendererGLRenderingVisitor::enterGLTriangleSet(GLTriangleSet *G) const
{
  if (G->getSolid())
  {
    glEnable(GL_CULL_FACE);
    if (G->getCcw())
      glFrontFace(GL_CCW);
    else
      glFrontFace(GL_CW);
    glCullFace(GL_BACK);   
  }
  
  if ((G->getColor()) && (G->getTexCoord()))
  {
  }
  
  if ((G->getColor()) && (!G->getTexCoord()))
  {
    glEnable(GL_COLOR_MATERIAL);
    glInterleavedArrays(GL_C4F_N3F_V3F, 0, &G->getC4F_N3F_V3FvertexTriangleArray().front());
    glDrawArrays(GL_TRIANGLES, 0, G->getC4F_N3F_V3FvertexTriangleArray().size());
    glDisable(GL_COLOR_MATERIAL);
  }
  
  if ((!G->getColor()) && (G->getTexCoord()))
  {
  }
  
  if ((!G->getColor()) && (!G->getTexCoord()))
  {
    glInterleavedArrays(GL_N3F_V3F, 0, &G->getN3F_V3FvertexTriangleArray().front());
    glDrawArrays(GL_TRIANGLES, 0, G->getN3F_V3FvertexTriangleArray().size());
  }
  
  if (G->getSolid())
    glDisable(GL_CULL_FACE);
}

void GLRendererGLRenderingVisitor::enterGLTriangleStripSet(GLTriangleStripSet *G) const
{
  if (G->getSolid())
  {
    glEnable(GL_CULL_FACE);
    if (G->getCcw())
      glFrontFace(GL_CCW);
    else
      glFrontFace(GL_CW);
    glCullFace(GL_BACK);   
  }
  
  if ((G->getColor()) && (G->getTexCoord()))
  {
  }
  
  if ((G->getColor()) && (!G->getTexCoord()))
  {
    glEnable(GL_COLOR_MATERIAL);
    if (!G->getVerticesDuplicated())
    {
      for (vector<vector<C4F_N3F_V3F> *>::const_iterator itStripArray = G->getC4F_N3F_V3FvertexStripArrayArray().begin(); itStripArray != G->getC4F_N3F_V3FvertexStripArrayArray().end(); ++itStripArray)
      {
        glInterleavedArrays(GL_C4F_N3F_V3F, 0, &(*itStripArray)->front());
        glDrawArrays(GL_TRIANGLE_STRIP, 0, (*itStripArray)->size());
      }
    }
    else
    {
      for (vector<vector<C4F_N3F_V3F> *>::const_iterator itStripArray = G->getC4F_N3F_V3FvertexStripArrayArray().begin(); itStripArray != G->getC4F_N3F_V3FvertexStripArrayArray().end(); ++itStripArray)
      {
        glInterleavedArrays(GL_C4F_N3F_V3F, 0, &(*itStripArray)->front());
        glDrawArrays(GL_TRIANGLES, 0, (*itStripArray)->size());
      } 
    } 
    glDisable(GL_COLOR_MATERIAL);
  }
  
  if ((!G->getColor()) && (G->getTexCoord()))
  {
  }
  
  if ((!G->getColor()) && (!G->getTexCoord()))
  {
    if (!G->getVerticesDuplicated())
    {
      for (vector<vector<N3F_V3F> *>::const_iterator itStripArray = G->getN3F_V3FvertexStripArrayArray().begin(); itStripArray != G->getN3F_V3FvertexStripArrayArray().end(); ++itStripArray)
      {
        glInterleavedArrays(GL_N3F_V3F, 0, &(*itStripArray)->front());
        glDrawArrays(GL_TRIANGLE_STRIP, 0, (*itStripArray)->size());
      }
    }
    else
    {
      for (vector<vector<N3F_V3F> *>::const_iterator itStripArray = G->getN3F_V3FvertexStripArrayArray().begin(); itStripArray != G->getN3F_V3FvertexStripArrayArray().end(); ++itStripArray)
      {
        glInterleavedArrays(GL_N3F_V3F, 0, &(*itStripArray)->front());
        glDrawArrays(GL_TRIANGLES, 0, (*itStripArray)->size());
      } 
    }  
  }
  
  if (G->getSolid())
    glDisable(GL_CULL_FACE);
}

