#include "GLUpdaterGLGeometry3DVisitor.h"
#include "GLBox.h"
#include "BoxDrawArray.h"
#include "GLCone.h"
#include "ConeDrawArray.h"
#include "GLCylinder.h"
#include "CylinderDrawArray.h"
#include "GLIndexedFaceSet.h"
#include "GLSphere.h"
#include "SphereDrawArray.h"
#include "Box.h"
#include "Cone.h"
#include "Cylinder.h"
#include "IndexedFaceSet.h"
#include "Sphere.h"
#include "Normal.h"
#include "Coordinate.h"
#include "TextureCoordinate.h"
#include "Color.h"
#include "ColorRGBA.h"

#include <vector>
#include <list>

using namespace X3DTK;
using namespace std;

GLUpdaterGLGeometry3DVisitor::GLUpdaterGLGeometry3DVisitor()
: GLGeometry3DVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLUpdaterGLGeometry3DVisitor, GLBox>(&GLUpdaterGLGeometry3DVisitor::enterGLBox);
  defineNewEnterFunction<GLUpdaterGLGeometry3DVisitor, GLCone>(&GLUpdaterGLGeometry3DVisitor::enterGLCone);
  defineNewEnterFunction<GLUpdaterGLGeometry3DVisitor, GLCylinder>(&GLUpdaterGLGeometry3DVisitor::enterGLCylinder);
  defineNewEnterFunction<GLUpdaterGLGeometry3DVisitor, GLIndexedFaceSet>(&GLUpdaterGLGeometry3DVisitor::enterGLIndexedFaceSet);
  defineNewEnterFunction<GLUpdaterGLGeometry3DVisitor, GLSphere>(&GLUpdaterGLGeometry3DVisitor::enterGLSphere);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLUpdaterGlobalVariables>();
}

GLUpdaterGLGeometry3DVisitor::~GLUpdaterGLGeometry3DVisitor()
{
}

void GLUpdaterGLGeometry3DVisitor::enterGLBox(GLBox *G) const
{
  Box *B = static_cast<Box *>(G->getLink());

  G->getSize() = B->getSize();
  G->getBoxDrawArray() = BoxDrawArray::getInstance();
}

void GLUpdaterGLGeometry3DVisitor::enterGLCone(GLCone *G) const
{
  Cone *C = static_cast<Cone *>(G->getLink());

  G->getBottomRadius() = C->getBottomRadius();
  G->getHeight() = C->getHeight();
  G->getSide() = C->getSide();
  G->getBottom() = C->getBottom();
  G->getConeDrawArray() = ConeDrawArray::getInstanceOfSection(64);
}

void GLUpdaterGLGeometry3DVisitor::enterGLCylinder(GLCylinder *G) const
{
  Cylinder *C = static_cast<Cylinder *>(G->getLink());
  
  G->getBottom() = C->getBottom();
  G->getRadius() = C->getRadius();
  G->getHeight() = C->getHeight();
  G->getSide() = C->getSide();
  G->getTop() = C->getTop();
  G->getCylinderDrawArray() = CylinderDrawArray::getInstanceOfSection(64);
}

void GLUpdaterGLGeometry3DVisitor::enterGLSphere(GLSphere *G) const
{
  Sphere *S = static_cast<Sphere *>(G->getLink());

  G->getRadius() = S->getRadius();
  G->getSphereDrawArray() = SphereDrawArray::getInstanceOfSection(64);  
}

void GLUpdaterGLGeometry3DVisitor::enterGLIndexedFaceSet(GLIndexedFaceSet *G) const
{
  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());  

  //for the moment if the node color has changed, all the attributes are computed
  //emptying the arrays
  G->getN3F_V3FvertexFaceArray().clear();
  G->getC4F_N3F_V3FvertexFaceArray().clear(); 
  G->getT2F_N3F_V3FvertexFaceArray().clear();
  G->getT2F_C4F_N3F_V3FvertexFaceArray().clear(); 
  G->getIndexFaceArray().clear();

  MFVec3f vertexArray;
  MFVec3f normalArray;
  MFInt32 coordIndex;

  Normal *NormalNode = static_cast<Normal *>(I->getNormal());

  if (NormalNode != 0)
    normalArray = NormalNode->getVector();

  //IndexedFaceSet attributes  
  if (I->getCoordIndex().empty())
    return;
 
  G->getSolid() = I->getSolid();
  G->getCcw() = I->getCcw();

  computeArrays(G, vertexArray, normalArray, coordIndex); 

  if ((I->getColor() != 0) && (I->getTexCoord() != 0))
  {
    G->getColor() = true;
    G->getTexCoord() = true;
    computeColor_TexCoord(G, vertexArray, normalArray, coordIndex);
  }
  if ((I->getColor() != 0) && (I->getTexCoord() == 0))
  {
    G->getColor() = true;
    G->getTexCoord() = false;
    computeColor_NoTexCoord(G, vertexArray, normalArray, coordIndex);
  }
  if ((I->getColor() == 0) && (I->getTexCoord() != 0))
  {
    G->getColor() = false;
    G->getTexCoord() = true;
    computeNoColor_TexCoord(G, vertexArray, normalArray, coordIndex);
  }
  if ((I->getColor() == 0) && (I->getTexCoord() == 0))
  {
    G->getColor() = false;
    G->getTexCoord() = false;
    computeNoColor_NoTexCoord(G, vertexArray, normalArray, coordIndex);
  }
}

void GLUpdaterGLGeometry3DVisitor::computeNoColor_NoTexCoord(GLIndexedFaceSet *G, const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex) const
{
  //cout << "IndexedFaceSet: no color, no texCoord" << endl;

  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());

  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == 0)
    return;

  SFBool normalPerVertex = I->getNormalPerVertex();
  MFInt32 colorIndex = I->getColorIndex(); 
  MFInt32 normalIndex = I->getNormalIndex(); 
  
  //using the GL_N3F_V3F
  //vertex array
  if ((normalPerVertex) 
   && (normalIndex.empty()))
  {
    //cout << "vertices not duplicated" << endl;
    
    G->getVerticesDuplicated() = false;
    //using the GL_POLYGON 
    //filling the vertex array
    MFVec3f::const_iterator itNormal = normalArray.begin();
    
    G->getN3F_V3FvertexFaceArray().resize(vertexArray.size());
    vector<N3F_V3F>::iterator itElement = G->getN3F_V3FvertexFaceArray().begin();
      
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++itCoord)
    {
      (*itElement).normal = *itNormal; 
      (*itElement).vertex = *itCoord;
      ++itNormal;
      ++itElement;
    }
      
    //filling the index array
    //iterate the coordIndex
    G->getIndexFaceArray().reserve(3*coordIndex.size()/4);
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    while (itCoordIndex != coordIndex.end())    
    {
      unsigned int first, a, b;
      
      first = *itCoordIndex;
      ++itCoordIndex;
      a = *itCoordIndex;
      ++itCoordIndex;
      
      while (*itCoordIndex != -1)
      {
        G->getIndexFaceArray().push_back(first);
        G->getIndexFaceArray().push_back(a);
        b = *itCoordIndex;
        ++itCoordIndex;
        G->getIndexFaceArray().push_back(b);
        a = b;
      }  
      ++itCoordIndex;
    }
  }
  else 
  {
    //cout << "vertices duplicated" << endl;
  
    G->getVerticesDuplicated() = true;
    //we cannot use the index array because an attribute is not constant per
    //instantiation of vertex
    //using the GL_POLYGON 
    //filling the arrays
    G->getN3F_V3FvertexFaceArray().reserve(3*coordIndex.size()/4);
        
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    MFInt32::const_iterator itNormalIndex;
    
    if (normalIndex.empty())
      itNormalIndex = coordIndex.begin();
    else
      itNormalIndex = normalIndex.begin();  
    
    while (itCoordIndex != coordIndex.end())    
    {
      N3F_V3F first, a, b;
      
      //computing first
      first.normal = normalArray[*itNormalIndex];
      first.vertex = vertexArray[*itCoordIndex];
           
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itCoordIndex;
      
      //computing a
      a.normal = normalArray[*itNormalIndex];
      a.vertex = vertexArray[*itCoordIndex];
           
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itCoordIndex;
      
      while (*itCoordIndex != -1)
      {
        //pushing back first, a     
        G->getN3F_V3FvertexFaceArray().push_back(first);
        G->getN3F_V3FvertexFaceArray().push_back(a);
        
        //computing b
        b.normal = normalArray[*itNormalIndex];
        b.vertex = vertexArray[*itCoordIndex];
           
        if (normalPerVertex)
          ++itNormalIndex;      
        ++itCoordIndex;
        
        //pushing back b 
        G->getN3F_V3FvertexFaceArray().push_back(b);
        
        //assigning b to a
        a.normal = b.normal;
        a.vertex = b.vertex;
      }
      ++itCoordIndex;
      ++itNormalIndex;
    }
  }
}

void GLUpdaterGLGeometry3DVisitor::computeColor_NoTexCoord(GLIndexedFaceSet *G, const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex) const
{
  //cout << "IndexedFaceSet: color, no texCoord" << endl;

  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());

  MFColorRGBA colorRGBA;
  if (I->getColor()->getTypeName() == "ColorRGBA")
    colorRGBA = static_cast<ColorRGBA *>(I->getColor())->getColor();
  else  
    colorRGBA = static_cast<Color *>(I->getColor())->getColor();
  
  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == 0)
    return;
  
  SFBool normalPerVertex = I->getNormalPerVertex();
  SFBool colorPerVertex = I->getColorPerVertex();
  MFInt32 colorIndex = I->getColorIndex(); 
  MFInt32 normalIndex = I->getNormalIndex(); 
  
  //using the GL_C4F_N3F_V3F
  //vertex array
  if ((normalPerVertex)
   && (colorPerVertex) 
   && (colorIndex.empty()) 
   && (normalIndex.empty()))
  {
    //cout << "vertices not duplicated" << endl;

    G->getVerticesDuplicated() = false;
    //using the GL_POLYGON 
    //filling the vertex array
    MFColorRGBA::const_iterator itColor = colorRGBA.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
    
    G->getC4F_N3F_V3FvertexFaceArray().resize(vertexArray.size());
    vector<C4F_N3F_V3F>::iterator itElement = G->getC4F_N3F_V3FvertexFaceArray().begin();
        
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++itCoord)
    {
      (*itElement).color = *itColor;
      (*itElement).normal = *itNormal; 
      (*itElement).vertex = *itCoord;
      ++itColor;
      ++itNormal;
      ++itElement;
    }
    //filling the index array
    //iterate the coordIndex
    G->getIndexFaceArray().reserve(3*coordIndex.size()/4);
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    while (itCoordIndex != coordIndex.end())    
    {
      unsigned int first, a, b;
      
      first = *itCoordIndex;
      ++itCoordIndex;
      a = *itCoordIndex;
      ++itCoordIndex;
      
      while (*itCoordIndex != -1)
      {
        G->getIndexFaceArray().push_back(first);
        G->getIndexFaceArray().push_back(a);
        b = *itCoordIndex;
        ++itCoordIndex;
        G->getIndexFaceArray().push_back(b);
        a = b;
      }  
      ++itCoordIndex;
    }
  }
  else //normalPerVertex or colorPertex is FALSE or ...
  {
    //cout << "vertices duplicated" << endl;

    G->getVerticesDuplicated() = true;
    //we cannot use the index array because an attribute is not constant per
    //instantiation of vertex
    //using the GL_POLYGON 
    //filling the vertex array
    //filling the arrays
    G->getC4F_N3F_V3FvertexFaceArray().reserve(3*coordIndex.size()/4);
        
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    MFInt32::const_iterator itColorIndex;
    MFInt32::const_iterator itNormalIndex;
    
    if (colorIndex.empty())
      itColorIndex = coordIndex.begin();
    else
      itColorIndex = colorIndex.begin();  
    
    if (normalIndex.empty())
      itNormalIndex = coordIndex.begin();
    else
      itNormalIndex = normalIndex.begin();  
    
    while (itCoordIndex != coordIndex.end())    
    {
      C4F_N3F_V3F first, a, b;
      
      //computing first
      first.color = colorRGBA[*itColorIndex];
      first.normal = normalArray[*itNormalIndex];
      first.vertex = vertexArray[*itCoordIndex];
      
      if (colorPerVertex)
        ++itColorIndex;           
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itCoordIndex;
      
      //computing a
      a.color = colorRGBA[*itColorIndex];
      a.normal = normalArray[*itNormalIndex];
      a.vertex = vertexArray[*itCoordIndex];
      
      if (colorPerVertex)
        ++itColorIndex;     
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itCoordIndex;
      
      while (*itCoordIndex != -1)
      {
        //pushing back first, a     
        G->getC4F_N3F_V3FvertexFaceArray().push_back(first);
        G->getC4F_N3F_V3FvertexFaceArray().push_back(a);
        
        //computing b
        b.color = colorRGBA[*itColorIndex];
        b.normal = normalArray[*itNormalIndex];
        b.vertex = vertexArray[*itCoordIndex];
        
        if (colorPerVertex)
          ++itColorIndex;   
        if (normalPerVertex)
          ++itNormalIndex;      
        ++itCoordIndex;
        
        //pushing back b 
        G->getC4F_N3F_V3FvertexFaceArray().push_back(b);
        
        //assigning b to a
        a.color = b.color;
        a.normal = b.normal;
        a.vertex = b.vertex;
      }
      if (!colorPerVertex)
          ++itColorIndex;  
      ++itCoordIndex;
      ++itNormalIndex;
    }
  }
}

void GLUpdaterGLGeometry3DVisitor::computeNoColor_TexCoord(GLIndexedFaceSet *G, const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex) const
{
  //cout << "IndexedFaceSet: no color, texCoord" << endl;

  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());

  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == 0)
    return;
  
  TextureCoordinate *TexCoordNode = static_cast<TextureCoordinate *>(I->getTexCoord());
  if (TexCoordNode == 0)
    return;
  MFVec2f texCoordArray = TexCoordNode->getPoint();
 
  SFBool normalPerVertex = I->getNormalPerVertex();
  MFInt32 colorIndex = I->getColorIndex(); 
  MFInt32 normalIndex = I->getNormalIndex(); 
  MFInt32 texCoordIndex = I->getTexCoordIndex(); 
  
  //using the GL_T2F_N3F_V3F
  //vertex array
  if ((normalPerVertex) 
   && (normalIndex.empty())
   && (texCoordIndex.empty()))
  {
    //cout << "vertices not duplicated" << endl;
  
    G->getVerticesDuplicated() = false;
    //using the GL_POLYGON 
    //filling the vertex array
    MFVec2f::const_iterator itTexCoord = texCoordArray.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    G->getT2F_N3F_V3FvertexFaceArray().resize(vertexArray.size());
    vector<T2F_N3F_V3F>::iterator itElement = G->getT2F_N3F_V3FvertexFaceArray().begin();
      
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++itCoord)
    {
      (*itElement).texCoord = *itTexCoord;
      (*itElement).normal = *itNormal; 
      (*itElement).vertex = *itCoord;
      ++itTexCoord;
      ++itNormal;
      ++itElement;
    }
      
    //filling the index array
    //iterate the coordIndex
    G->getIndexFaceArray().reserve(3*coordIndex.size()/4);
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    while (itCoordIndex != coordIndex.end())    
    {
      unsigned int first, a, b;
      
      first = *itCoordIndex;
      ++itCoordIndex;
      a = *itCoordIndex;
      ++itCoordIndex;
      
      while (*itCoordIndex != -1)
      {
        G->getIndexFaceArray().push_back(first);
        G->getIndexFaceArray().push_back(a);
        b = *itCoordIndex;
        ++itCoordIndex;
        G->getIndexFaceArray().push_back(b);
        a = b;
      }  
      ++itCoordIndex;
    }
  }
  else //normalPerVertex is FALSE or ...
  {
    //cout << "vertices duplicated" << endl;
  
    G->getVerticesDuplicated() = true;
    //we cannot use the index array because an attribute is not constant per
    //instantiation of vertex
    //using the GL_POLYGON 
    //filling the arrays
    G->getT2F_N3F_V3FvertexFaceArray().reserve(3*coordIndex.size()/4);
        
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    MFInt32::const_iterator itNormalIndex;
    MFInt32::const_iterator itTexCoordIndex;
    
    if (normalIndex.empty())
      itNormalIndex = coordIndex.begin();
    else
      itNormalIndex = normalIndex.begin();  

    if (texCoordIndex.empty())
      itTexCoordIndex = coordIndex.begin();
    else
      itTexCoordIndex = texCoordIndex.begin();  

    
    while (itCoordIndex != coordIndex.end())    
    {
      T2F_N3F_V3F first, a, b;
      
      //computing first
      first.normal = normalArray[*itNormalIndex];
      first.vertex = vertexArray[*itCoordIndex];
      first.texCoord = texCoordArray[*itTexCoordIndex];
      
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itCoordIndex;
      ++itTexCoordIndex;
      
      //computing a
      a.normal = normalArray[*itNormalIndex];
      a.vertex = vertexArray[*itCoordIndex];
      a.texCoord = texCoordArray[*itTexCoordIndex];
      
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itCoordIndex;
      ++itTexCoordIndex;
      
      while (*itCoordIndex != -1)
      {
        //pushing back first, a     
        G->getT2F_N3F_V3FvertexFaceArray().push_back(first);
        G->getT2F_N3F_V3FvertexFaceArray().push_back(a);
        
        //computing b
        b.normal = normalArray[*itNormalIndex];
        b.vertex = vertexArray[*itCoordIndex];
        b.texCoord = texCoordArray[*itTexCoordIndex];
              
        if (normalPerVertex)
          ++itNormalIndex;      
        ++itCoordIndex;
        ++itTexCoordIndex;
        
        //pushing back b 
        G->getT2F_N3F_V3FvertexFaceArray().push_back(b);
        
        //assigning b to a
        a.normal = b.normal;
        a.vertex = b.vertex;
        a.texCoord = b.texCoord;
      }
      ++itCoordIndex;
      ++itNormalIndex;
      ++itTexCoordIndex;
    }
  }
}

void GLUpdaterGLGeometry3DVisitor::computeColor_TexCoord(GLIndexedFaceSet *G, const MFVec3f &vertexArray, const MFVec3f &normalArray, const MFInt32 &coordIndex) const
{
  //cout << "IndexedFaceSet: color, texCoord" << endl;
  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());

  MFColorRGBA colorRGBA;
  if (I->getColor()->getTypeName() == "ColorRGBA")
    colorRGBA = static_cast<ColorRGBA *>(I->getColor())->getColor();
  else  
    colorRGBA = static_cast<Color *>(I->getColor())->getColor();
  
  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == 0)
    return;
  
  TextureCoordinate *TexCoordNode = static_cast<TextureCoordinate *>(I->getTexCoord());
  if (TexCoordNode == 0)
    return;
  MFVec2f texCoordArray = TexCoordNode->getPoint();
  
  SFBool normalPerVertex = I->getNormalPerVertex();
  SFBool colorPerVertex = I->getColorPerVertex();
  MFInt32 colorIndex = I->getColorIndex(); 
  MFInt32 normalIndex = I->getNormalIndex(); 
  MFInt32 texCoordIndex = I->getCoordIndex(); 
  
  //using the GL_T2F_C4F_N3F_V3F
  //vertex array
  if ((normalPerVertex)
   && (colorPerVertex) 
   && (colorIndex.empty()) 
   && (normalIndex.empty())
   && (texCoordIndex.empty()))
  {
    //cout << "vertices not duplicated" << endl;

    G->getVerticesDuplicated() = false;
    //using the GL_POLYGON 
    //filling the vertex array
    MFColorRGBA::const_iterator itColor = colorRGBA.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
    MFVec2f::const_iterator itTexCoord = texCoordArray.begin();
    
    G->getT2F_C4F_N3F_V3FvertexFaceArray().resize(vertexArray.size());
    vector<T2F_C4F_N3F_V3F>::iterator itElement = G->getT2F_C4F_N3F_V3FvertexFaceArray().begin();
        
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++itCoord)
    {
      (*itElement).color = *itColor;
      (*itElement).normal = *itNormal; 
      (*itElement).vertex = *itCoord;
      (*itElement).texCoord = *itTexCoord;
      ++itColor;
      ++itNormal;
      ++itTexCoord;
      ++itElement;
    }
    //filling the index array
    //iterate the coordIndex
    G->getIndexFaceArray().reserve(3*coordIndex.size()/4);
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    while (itCoordIndex != coordIndex.end())    
    {
      unsigned int first, a, b;
      
      first = *itCoordIndex;
      ++itCoordIndex;
      a = *itCoordIndex;
      ++itCoordIndex;
      
      while (*itCoordIndex != -1)
      {
        G->getIndexFaceArray().push_back(first);
        G->getIndexFaceArray().push_back(a);
        b = *itCoordIndex;
        ++itCoordIndex;
        G->getIndexFaceArray().push_back(b);
        a = b;
      }  
      ++itCoordIndex;
    }
  }
  else //normalPerVertex or colorPertex is FALSE or ...
  {

    //cout << "vertices duplicated" << endl;

    G->getVerticesDuplicated() = true;
    //we cannot use the index array because an attribute is not constant per
    //instantiation of vertex
    //using the GL_POLYGON 
    //filling the vertex array
    //filling the arrays
    G->getT2F_C4F_N3F_V3FvertexFaceArray().reserve(3*coordIndex.size()/4);
        
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    MFInt32::const_iterator itColorIndex;
    MFInt32::const_iterator itNormalIndex;
    MFInt32::const_iterator itTexCoordIndex;
    
    if (colorIndex.empty())
      itColorIndex = coordIndex.begin();
    else
      itColorIndex = colorIndex.begin();  
    
    if (normalIndex.empty())
      itNormalIndex = coordIndex.begin();
    else
      itNormalIndex = normalIndex.begin();  
    
    if (texCoordIndex.empty())
      itTexCoordIndex = coordIndex.begin();
    else
      itTexCoordIndex = texCoordIndex.begin(); 
    
    while (itCoordIndex != coordIndex.end())    
    {
      T2F_C4F_N3F_V3F first, a, b;
      
      //computing first
      first.color = colorRGBA[*itColorIndex];
      first.normal = normalArray[*itNormalIndex];
      first.vertex = vertexArray[*itCoordIndex];
      first.texCoord = texCoordArray[*itTexCoordIndex];
      
      if (colorPerVertex)
        ++itColorIndex;           
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itCoordIndex;
      ++itTexCoordIndex;
      
      //computing a
      a.color = colorRGBA[*itColorIndex];
      a.normal = normalArray[*itNormalIndex];
      a.vertex = vertexArray[*itCoordIndex];
      a.texCoord = texCoordArray[*itTexCoordIndex];
      
      if (colorPerVertex)
        ++itColorIndex;     
      if (normalPerVertex)
        ++itNormalIndex;      
      ++itCoordIndex;
      ++itTexCoordIndex;
      
      while (*itCoordIndex != -1)
      {
        //pushing back first, a     
        G->getT2F_C4F_N3F_V3FvertexFaceArray().push_back(first);
        G->getT2F_C4F_N3F_V3FvertexFaceArray().push_back(a);
        
        //computing b
        b.color = colorRGBA[*itColorIndex];
        b.normal = normalArray[*itNormalIndex];
        b.vertex = vertexArray[*itCoordIndex];
        b.texCoord = texCoordArray[*itTexCoordIndex];
        
        if (colorPerVertex)
          ++itColorIndex;   
        if (normalPerVertex)
          ++itNormalIndex;      
        ++itCoordIndex;
        ++itTexCoordIndex;
        
        //pushing back b 
        G->getT2F_C4F_N3F_V3FvertexFaceArray().push_back(b);
        
        //assigning b to a
        a.color = b.color;
        a.normal = b.normal;
        a.vertex = b.vertex;
        a.texCoord = b.texCoord;
      }
      if (!colorPerVertex)
        ++itColorIndex;  
      ++itCoordIndex;
      ++itNormalIndex;
      ++itTexCoordIndex;
    }
  }
}

void GLUpdaterGLGeometry3DVisitor::computeArrays(GLIndexedFaceSet *G, MFVec3f &vertexArray, MFVec3f &normalArray, MFInt32 &coordIndex) const
{
  IndexedFaceSet *I = static_cast<IndexedFaceSet *>(G->getLink());
  
  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == 0)
    return;

  if (!normalArray.empty())
  {
    //no process is necessary
    vertexArray = CoordNode->getPoint();
    coordIndex = I->getCoordIndex();
    return;
  }
  
  coordIndex = I->getCoordIndex();
  vertexArray = CoordNode->getPoint();
    
  SFBool normalPerVertex = I->getNormalPerVertex();
  coordIndex = I->getCoordIndex();
  
  //computing the array containing the normals of each face
  MFVec3f normalFaceArray;
  MFInt32::const_iterator itCoordIndex = coordIndex.begin();
  
  while (itCoordIndex != coordIndex.end())
  {
    SFVec3f A = vertexArray[*itCoordIndex];
    ++itCoordIndex;
    SFVec3f B = vertexArray[*itCoordIndex];
    ++itCoordIndex;
    SFVec3f C = vertexArray[*itCoordIndex];
    ++itCoordIndex;

    //computing the normal following ccw
    SFVec3f AB = (B - A).normalize();
    SFVec3f AC = (C - A).normalize();
    SFVec3f N;

    if (G->getCcw())
      N = crossprod(AB, AC);
    else
      N = crossprod(AC, AB);  

    normalFaceArray.push_back(N);
    ++itCoordIndex;//the pointed value is -1
  }

  if (normalPerVertex)
  {
    //for the moment crease angle is not used
    normalArray = MFVec3f(vertexArray.size());
    
    //iterate the faces
    itCoordIndex = coordIndex.begin();
    MFVec3f::const_iterator itNormal = normalFaceArray.begin();
    
    while (itCoordIndex != coordIndex.end())
    {
      normalArray[*itCoordIndex] = normalArray[*itCoordIndex] + *itNormal;
      ++itCoordIndex;
      normalArray[*itCoordIndex] = normalArray[*itCoordIndex] + *itNormal;
      ++itCoordIndex;
      normalArray[*itCoordIndex] = normalArray[*itCoordIndex] + *itNormal;
      ++itCoordIndex;
      ++itCoordIndex;
      ++itNormal;
    }

    //then normalize
    for (MFVec3f::iterator itNormal = normalArray.begin(); itNormal != normalArray.end(); ++itNormal)
    {
      (*itNormal).normalize();
    }
  }
  else
  {
    normalArray = MFVec3f();
    //copying the values of normalFaceArray
    for (MFVec3f::const_iterator itFace = normalFaceArray.begin(); itFace != normalFaceArray.end(); ++itFace)
      normalArray.push_back(*itFace);
  }
}

