#include "GLUpdaterGLGroupingVisitor.h"
#include "GLGroup.h"
#include "GLStaticGroup.h"
#include "GLTransform.h"
#include "Group.h"
#include "StaticGroup.h"
#include "Transform.h"

using namespace X3DTK;

GLUpdaterGLGroupingVisitor::GLUpdaterGLGroupingVisitor()
: GLGroupingVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLUpdaterGLGroupingVisitor, GLTransform>(&GLUpdaterGLGroupingVisitor::enterGLTransform);

  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLUpdaterGlobalVariables>();
}

GLUpdaterGLGroupingVisitor::~GLUpdaterGLGroupingVisitor()
{
}

void GLUpdaterGLGroupingVisitor::enterGLTransform(GLTransform *G) const
{
  Transform *T = static_cast<Transform *>(G->getLink());

  //computing the transformMatrix
  SFMatrix34f Tr = translation(T->getTranslation());
  SFMatrix34f C = translation(T->getCenter());
  SFVec3f v(T->getRotation().x, T->getRotation().y, T->getRotation().z);
  SFMatrix34f R = rotation(T->getRotation().angle, v);
  SFVec3f vs(T->getScaleOrientation().x, T->getScaleOrientation().y, T->getScaleOrientation().z);
  SFMatrix34f SR = rotation(T->getScaleOrientation().angle, vs);
  SFMatrix34f S = scale34(T->getScale().x, T->getScale().y, T->getScale().z);
  SFMatrix34f iSR = rotation(-T->getScaleOrientation().angle, vs);
  SFMatrix34f iC = translation(-1.0f*T->getCenter());
  SFMatrix34f transform = Tr*C*R*SR*S*iSR*iC;
   
  transform.toFloat16(G->getTransformMatrix());
}
