#include "GLUpdaterGLRenderingVisitor.h"
#include "GLIndexedLineSet.h"
#include "GLPointSet.h"
#include "GLTriangleFanSet.h"
#include "GLTriangleSet.h"
#include "GLTriangleStripSet.h"

#include "RenderingNodes.h"

#include <vector>

using namespace X3DTK;
using namespace std;

GLUpdaterGLRenderingVisitor::GLUpdaterGLRenderingVisitor()
: GLRenderingVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLUpdaterGLRenderingVisitor, GLIndexedLineSet>(&GLUpdaterGLRenderingVisitor::enterGLIndexedLineSet);
  defineNewEnterFunction<GLUpdaterGLRenderingVisitor, GLPointSet>(&GLUpdaterGLRenderingVisitor::enterGLPointSet);
  defineNewEnterFunction<GLUpdaterGLRenderingVisitor, GLTriangleFanSet>(&GLUpdaterGLRenderingVisitor::enterGLTriangleFanSet);
  defineNewEnterFunction<GLUpdaterGLRenderingVisitor, GLTriangleSet>(&GLUpdaterGLRenderingVisitor::enterGLTriangleSet);
  defineNewEnterFunction<GLUpdaterGLRenderingVisitor, GLTriangleStripSet>(&GLUpdaterGLRenderingVisitor::enterGLTriangleStripSet);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLUpdaterGlobalVariables>();
}

GLUpdaterGLRenderingVisitor::~GLUpdaterGLRenderingVisitor()
{
}

void GLUpdaterGLRenderingVisitor::enterGLIndexedLineSet(GLIndexedLineSet *G) const
{
  IndexedLineSet *I = static_cast<IndexedLineSet *>(G->getLink());
  
  //emptying the arrays
  G->getPointArray() = MFVec3f();
  for (vector<vector<C4UB_V3F> *>::const_iterator itLineArray = G->getLineArrayArray().begin(); itLineArray != G->getLineArrayArray().end(); ++itLineArray)
    delete (*itLineArray);
  G->getLineArrayArray() = vector<vector<C4UB_V3F> *>();  
  
  for (vector<MFInt32 *>::const_iterator itLineIndex = G->getLineIndexArray().begin(); itLineIndex != G->getLineIndexArray().end(); ++itLineIndex)
    delete (*itLineIndex);
  G->getLineIndexArray() = vector<MFInt32 *>();

  Coordinate *CoordNode = static_cast<Coordinate *>(I->getCoord());
  if (CoordNode == 0)
    return;
  G->getPointArray() = CoordNode->getPoint();

  MFInt32 coordIndex = I->getCoordIndex();
  G->getLineWidth() = I->getLineWidth();
  
  if (I->getColor() != 0)
  {
    G->getColor() = true;
    MFColorRGBA colorRGBA = static_cast<ColorRGBA *>(I->getColor())->getColor();
    G->getColorPerVertex() = I->getColorPerVertex();
    MFInt32 colorIndex = I->getColorIndex();
  
    if (G->getColorPerVertex())
    {
      //we use line_strip but the points are duplicated because of the fact 
      //that a same point can be of two colors
      //there is one array per continuous line
      //filling the vertex arrays
      MFInt32::const_iterator itCoordIndex = coordIndex.begin();
         
      if (!colorIndex.empty())
      {
        MFInt32::const_iterator itColorIndex = colorIndex.begin();  
          
        while (itCoordIndex != coordIndex.end())
        {
          vector<C4UB_V3F> *lineArray = new vector<C4UB_V3F>();
          while ((itCoordIndex != coordIndex.end()) && (*itCoordIndex != -1))
          {
            C4UB_V3F element;
            SFColorRGBA currentColor = colorRGBA[*itColorIndex];
            element.r = (unsigned char)floorf(255.0f*currentColor.r);
            element.g = (unsigned char)floorf(255.0f*currentColor.g);
            element.b = (unsigned char)floorf(255.0f*currentColor.b);
            element.a = (unsigned char)floorf(255.0f*currentColor.a);
            element.vertex = G->getPointArray()[*itCoordIndex];
            lineArray->push_back(element);
            ++itCoordIndex;
            ++itColorIndex;
          }
          G->getLineArrayArray().push_back(lineArray);
          ++itCoordIndex;
        }
      }
      else
      {
        while (itCoordIndex != coordIndex.end())
        {
          vector<C4UB_V3F> *lineArray = new vector<C4UB_V3F>();
          while ((itCoordIndex != coordIndex.end()) && (*itCoordIndex != -1))
          {
            C4UB_V3F element;
            SFColorRGBA currentColor = colorRGBA[*itCoordIndex];
            element.r = (unsigned char)floorf(255.0f*currentColor.r);
            element.g = (unsigned char)floorf(255.0f*currentColor.g);
            element.b = (unsigned char)floorf(255.0f*currentColor.b);
            element.a = (unsigned char)floorf(255.0f*currentColor.a);
            element.vertex = G->getPointArray()[*itCoordIndex];
        
            lineArray->push_back(element);
            ++itCoordIndex;
          }
          G->getLineArrayArray().push_back(lineArray);
          ++itCoordIndex;
        }
      }
    }
    else
    {
      //the colors are applies to the segments, so we define a large array where each 
      //point is duplicated and use independant lines
      //reading the content of the coordIndex
      G->getColor() = false;
      MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    
      if (!colorIndex.empty())
      {
        MFInt32::const_iterator itColorIndex = colorIndex.begin(); 
        
        while (itCoordIndex != coordIndex.end())
        {
          unsigned int precCoordIndex = *itCoordIndex;
          ++itCoordIndex;
          if (itCoordIndex == coordIndex.end())
            break;
          
          if (*itCoordIndex != -1)
          {
            C4UB_V3F element;
            SFColorRGBA currentColor = colorRGBA[*itColorIndex];
            element.r = (unsigned char)floorf(255.0f*currentColor.r);
            element.g = (unsigned char)floorf(255.0f*currentColor.g);
            element.b = (unsigned char)floorf(255.0f*currentColor.b);
            element.a = (unsigned char)floorf(255.0f*currentColor.a);
            element.vertex = G->getPointArray()[precCoordIndex];
            ++itColorIndex;
            //we push back the two elements keeping the color
            G->getPointLineArray().push_back(element);
            
            element.vertex = G->getPointArray()[*itCoordIndex];
            G->getPointLineArray().push_back(element);
          }   
          else
          {
            ++itCoordIndex;
          }  
        }
      }
      else
      {
        while (itCoordIndex != coordIndex.end())
        {
          unsigned int precCoordIndex = *itCoordIndex;
          ++itCoordIndex;
          if (itCoordIndex == coordIndex.end())
            break;
          
          if (*itCoordIndex != -1)
          {
            C4UB_V3F element;
            SFColorRGBA currentColor = colorRGBA[*itCoordIndex];
            element.r = (unsigned char)floorf(255.0f*currentColor.r);
            element.g = (unsigned char)floorf(255.0f*currentColor.g);
            element.b = (unsigned char)floorf(255.0f*currentColor.b);
            element.a = (unsigned char)floorf(255.0f*currentColor.a);
            element.vertex = G->getPointArray()[precCoordIndex];
          
            //we push back the two elements keeping the color
            G->getPointLineArray().push_back(element);
            
            element.vertex = G->getPointArray()[*itCoordIndex];
            G->getPointLineArray().push_back(element);
          }   
          else
          {
            ++itCoordIndex;
          }  
        }
      }
    }
  }
  else
  {
    //the material properties are applied
    //defining the set of indices
    //reading the content of the coordIndex
    MFInt32::const_iterator itCoordIndex = coordIndex.begin();
    while (itCoordIndex != coordIndex.end())
    {
      MFInt32 *lineIndex = new MFInt32();
      while ((itCoordIndex != coordIndex.end()) && (*itCoordIndex != -1))
      {
        lineIndex->push_back(*itCoordIndex);
        ++itCoordIndex;
      }
      G->getLineIndexArray().push_back(lineIndex);
      ++itCoordIndex;
    }
  }
}

void GLUpdaterGLRenderingVisitor::enterGLPointSet(GLPointSet *G) const
{
  PointSet *P = static_cast<PointSet *>(G->getLink());

  // emptying the arrays
  G->getPointArray() = MFVec3f();
  G->getPointSetArray() = vector<C4UB_V3F>();

  Coordinate *CoordNode = static_cast<Coordinate *>(P->getCoord());  
  if (CoordNode == 0)
    return;

  G->getPointArray() = CoordNode->getPoint();

  if (P->getColor() != 0)
  {
    MFColorRGBA colorRGBA = static_cast<ColorRGBA *>(P->getColor())->getColor();
      
    MFColorRGBA::const_iterator itColor = colorRGBA.begin();
    
    for (MFVec3f::const_iterator itPoint = G->getPointArray().begin(); itPoint != G->getPointArray().end(); ++itPoint)
    {
      C4UB_V3F element;
      SFColorRGBA currentColor = *itColor;
      element.r = (unsigned char)floorf(255.0f*currentColor.r);
      element.g = (unsigned char)floorf(255.0f*currentColor.g);
      element.b = (unsigned char)floorf(255.0f*currentColor.b);
      element.a = (unsigned char)floorf(255.0f*currentColor.a);
      element.vertex = *itPoint;
      G->getPointSetArray().push_back(element);
      ++itColor;
    }
  }
}

void GLUpdaterGLRenderingVisitor::enterGLTriangleFanSet(GLTriangleFanSet *G) const
{
  TriangleFanSet *T = static_cast<TriangleFanSet *>(G->getLink());
  
  //emptying the arrays
  for (vector<vector<N3F_V3F> *>::const_iterator itFanArray = G->getN3F_V3FvertexFanArrayArray().begin(); itFanArray != G->getN3F_V3FvertexFanArrayArray().end(); ++itFanArray)
    delete (*itFanArray);
  G->getN3F_V3FvertexFanArrayArray() = vector<vector<N3F_V3F> *>();

  for (vector<vector<C4F_N3F_V3F> *>::const_iterator itFanArray = G->getC4F_N3F_V3FvertexFanArrayArray().begin(); itFanArray != G->getC4F_N3F_V3FvertexFanArrayArray().end(); ++itFanArray)
    delete (*itFanArray);
  G->getC4F_N3F_V3FvertexFanArrayArray() = vector<vector<C4F_N3F_V3F> *>(); 

  Normal *NormalNode = static_cast<Normal *>(T->getNormal());
  MFVec3f normalArray;
  if (NormalNode != 0)
    normalArray = NormalNode->getVector();

  G->getSolid() = T->getSolid();
  G->getCcw() = T->getCcw();

  //computing normals if necessary
  if (normalArray.empty())
    computeNormalArray(G, normalArray);
    
  if ((T->getColor() != 0) && (T->getTexCoord() != 0))
  {
    G->getColor() = true;
    G->getTexCoord() = true;
    computeColor_TexCoord(G, normalArray);
  }  
  
  if ((T->getColor() != 0) && (T->getTexCoord() == 0))
  {
    G->getColor() = true;
    G->getTexCoord() = false;
    computeColor_NoTexCoord(G, normalArray);
  }  
  
  if ((T->getColor() == 0) && (T->getTexCoord() != 0))
  {
    G->getColor() = false;
    G->getTexCoord() = true;
    computeNoColor_TexCoord(G, normalArray);
  }  

  if ((T->getColor() == 0) && (T->getTexCoord() == 0))
  {
    G->getColor() = false;
    G->getTexCoord() = false;
    computeNoColor_NoTexCoord(G, normalArray);
  }
}

void GLUpdaterGLRenderingVisitor::enterGLTriangleSet(GLTriangleSet *G) const
{
  TriangleSet *T = static_cast<TriangleSet *>(G->getLink());

  //emptying arrays
  G->getN3F_V3FvertexTriangleArray() = vector<N3F_V3F>();
  G->getC4F_N3F_V3FvertexTriangleArray() = vector<C4F_N3F_V3F>();

  Normal *NormalNode = static_cast<Normal *>(T->getNormal());
  MFVec3f normalArray;
  if (NormalNode != 0)
    normalArray = NormalNode->getVector();

  G->getSolid() = T->getSolid();
  G->getCcw() = T->getCcw();

  //computing normals if necessary
  if (normalArray.empty())
    computeNormalArray(G, normalArray);

  if ((T->getColor() != 0) && (T->getTexCoord() != 0))
  {
    G->getColor() = true;
    G->getTexCoord() = true;
    computeColor_TexCoord(G, normalArray);
  }  
  
  if ((T->getColor() != 0) && (T->getTexCoord() == 0))
  {
    G->getColor() = true;
    G->getTexCoord() = false;
    computeColor_NoTexCoord(G, normalArray);
  }  
  
  if ((T->getColor() == 0) && (T->getTexCoord() != 0))
  {
    G->getColor() = false;
    G->getTexCoord() = true;
    computeNoColor_TexCoord(G, normalArray);
  }  

  if ((T->getColor() == 0) && (T->getTexCoord() == 0))
  {
    G->getColor() = false;
    G->getTexCoord() = false;
    computeNoColor_NoTexCoord(G, normalArray);
  }
}

void GLUpdaterGLRenderingVisitor::enterGLTriangleStripSet(GLTriangleStripSet *G) const
{
  TriangleStripSet *T = static_cast<TriangleStripSet *>(G->getLink());

  //emptying the arrays
  for (vector<vector<N3F_V3F> *>::const_iterator itStripArray = G->getN3F_V3FvertexStripArrayArray().begin(); itStripArray != G->getN3F_V3FvertexStripArrayArray().end(); ++itStripArray)
    delete (*itStripArray);
  G->getN3F_V3FvertexStripArrayArray() = vector<vector<N3F_V3F> *>();
 
  for (vector<vector<C4F_N3F_V3F> *>::const_iterator itStripArray = G->getC4F_N3F_V3FvertexStripArrayArray().begin(); itStripArray != G->getC4F_N3F_V3FvertexStripArrayArray().end(); ++itStripArray)
  delete (*itStripArray);
  G->getC4F_N3F_V3FvertexStripArrayArray() = vector<vector<C4F_N3F_V3F> *>(); 

  Normal *NormalNode = static_cast<Normal *>(T->getNormal());
  MFVec3f normalArray;
  if (NormalNode != 0)
    normalArray = NormalNode->getVector();

  G->getSolid() = T->getSolid();
  G->getCcw() = T->getCcw();

  //computing normals if necessary
  if (normalArray.empty())
    computeNormalArray(G, normalArray);

  if ((T->getColor() != 0) && (T->getTexCoord() != 0))
  {
    G->getColor() = true;
    G->getTexCoord() = true;
    computeColor_TexCoord(G, normalArray);
  }  
  
  if ((T->getColor() != 0) && (T->getTexCoord() == 0))
  {
    G->getColor() = true;
    G->getTexCoord() = false;
    computeColor_NoTexCoord(G, normalArray);
  }  
  
  if ((T->getColor() == 0) && (T->getTexCoord() != 0))
  {
    G->getColor() = false;
    G->getTexCoord() = true;
    computeNoColor_TexCoord(G, normalArray);
  }  

  if ((T->getColor() == 0) && (T->getTexCoord() == 0))
  {
    G->getColor() = false;
    G->getTexCoord() = false;
    computeNoColor_NoTexCoord(G, normalArray);
  } 
}

void GLUpdaterGLRenderingVisitor::computeNoColor_NoTexCoord(GLTriangleFanSet *G, const MFVec3f &normalArray) const
{
  TriangleFanSet *T = static_cast<TriangleFanSet *>(G->getLink());
  
  Coordinate *CoordNode = static_cast<Coordinate *>(T->getCoord());
  if (CoordNode == 0)
    return;
  MFVec3f vertexArray = CoordNode->getPoint();

  SFBool normalPerVertex = T->getNormalPerVertex();
  MFInt32 fanCount = T->getFanCount();
    
  if (normalPerVertex)
  {
    G->getVerticesDuplicated() = false;
    //using the GL_TRIANGLE_FAN
    MFVec3f::const_iterator itCoord = vertexArray.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
       
    //iterate the fans
    for (MFInt32::const_iterator itFan = fanCount.begin(); itFan != fanCount.end(); ++ itFan)
    {
      //filling a new array
      vector<N3F_V3F> *vertexFanArray = new vector<N3F_V3F>();
      for (int i = 0; i < *itFan; ++i)
      {
        N3F_V3F element;  
        element.normal = *itNormal;
        element.vertex = *itCoord;
        vertexFanArray->push_back(element);
            
        ++itCoord;
        ++itNormal;
      }
      G->getN3F_V3FvertexFanArrayArray().push_back(vertexFanArray);
    }
  }
  else //normalPerVertex is FALSE
  {
    G->getVerticesDuplicated() = true;
    //using the GL_TRIANGLES
    MFVec3f::const_iterator itCoord = vertexArray.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    //iterate the fans
    for (MFInt32::const_iterator itFan = fanCount.begin(); itFan != fanCount.end(); ++ itFan)
    {
      //filling a new array
      vector<N3F_V3F> *vertexFanArray = new vector<N3F_V3F>();
      N3F_V3F center;
      center.normal = *itNormal;
      center.vertex = *itCoord;
      ++itCoord;
        
      N3F_V3F element;  
      element.normal = *itNormal;
      element.vertex = *itCoord;

      for (int i = 2; i < *itFan; ++i)
      {
        //push center
        center.normal = *itNormal;
        vertexFanArray->push_back(center);
 
        //push Pi
        element.normal = *itNormal;
        vertexFanArray->push_back(element);
         
        ++itCoord;
         
        //push Pi+1  
        element.normal = *itNormal;
        element.vertex = *itCoord;
        vertexFanArray->push_back(element);
           
        ++itNormal;  
      }
      G->getN3F_V3FvertexFanArrayArray().push_back(vertexFanArray);
    }
  } 
}

void GLUpdaterGLRenderingVisitor::computeColor_NoTexCoord(GLTriangleFanSet *G, const MFVec3f &normalArray) const
{
  TriangleFanSet *T = static_cast<TriangleFanSet *>(G->getLink());
  
  MFColorRGBA colorRGBA = static_cast<ColorRGBA *>(T->getColor())->getColor();
  
  Coordinate *CoordNode = static_cast<Coordinate *>(T->getCoord());
  if (CoordNode == 0)
    return;
  MFVec3f vertexArray = CoordNode->getPoint();

  SFBool normalPerVertex = T->getNormalPerVertex();
  SFBool colorPerVertex = T->getColorPerVertex();
  MFInt32 fanCount = T->getFanCount();
    
  if ((normalPerVertex) && (colorPerVertex))
  {
    G->getVerticesDuplicated() = false;
    //using the GL_TRIANGLE_FAN
    MFColorRGBA::const_iterator itColor = colorRGBA.begin();
    MFVec3f::const_iterator itCoord = vertexArray.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    //iterate the fans
    for (MFInt32::const_iterator itFan = fanCount.begin(); itFan != fanCount.end(); ++ itFan)
    {
      //filling a new array
      vector<C4F_N3F_V3F> *vertexFanArray = new vector<C4F_N3F_V3F>();
      for (int i = 0; i < *itFan; ++i)
      {
        C4F_N3F_V3F element;  
        element.color = *itColor;
        element.normal = *itNormal;
        element.vertex = *itCoord;
        vertexFanArray->push_back(element);
          
        ++itColor;
        ++itCoord;
        ++itNormal;
      }
      G->getC4F_N3F_V3FvertexFanArrayArray().push_back(vertexFanArray);
    }
  }
  else //normalPerVertex or colorPertex is FALSE
  {
    G->getVerticesDuplicated() = true;
    //using the GL_TRIANGLES
    MFColorRGBA::const_iterator itColor = colorRGBA.begin();
    MFVec3f::const_iterator itCoord = vertexArray.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    //iterate the fans
    for (MFInt32::const_iterator itFan = fanCount.begin(); itFan != fanCount.end(); ++ itFan)
    {
      //filling a new array
      vector<C4F_N3F_V3F> *vertexFanArray = new vector<C4F_N3F_V3F>();
      C4F_N3F_V3F center;
      center.color = *itColor;
      center.normal = *itNormal;
      center.vertex = *itCoord;
 
      ++itCoord;
      if (colorPerVertex)
        ++itColor;
      if (normalPerVertex)
        ++itNormal;  
      C4F_N3F_V3F element;  
      element.color = *itColor;
      element.normal = *itNormal;
      element.vertex = *itCoord;

      for (int i = 2; i < *itFan; ++i)
      {
        //push center
        if (!colorPerVertex)
          center.color = *itColor;
        if (!normalPerVertex)
          center.normal = *itNormal;
        vertexFanArray->push_back(center);
 
        //push Pi
        if (!colorPerVertex)
          element.color = *itColor;
        if (!normalPerVertex)  
          element.normal = *itNormal;
        vertexFanArray->push_back(element);
           
        ++itCoord;
        if (colorPerVertex)
          ++itColor;
        if (normalPerVertex)
          ++itNormal;  
            
        //push Pi+1  
        element.color = *itColor;
        element.normal = *itNormal;
        element.vertex = *itCoord;
        vertexFanArray->push_back(element);
          
        if (!colorPerVertex)
          ++itColor;
        if (!normalPerVertex)
          ++itNormal;  
      }
      G->getC4F_N3F_V3FvertexFanArrayArray().push_back(vertexFanArray);
    }
  } 
}

void GLUpdaterGLRenderingVisitor::computeNoColor_TexCoord(GLTriangleFanSet *G, const MFVec3f &normalArray) const
{
}

void GLUpdaterGLRenderingVisitor::computeColor_TexCoord(GLTriangleFanSet *G, const MFVec3f &normalArray) const
{
}

void GLUpdaterGLRenderingVisitor::computeNormalArray(GLTriangleFanSet *G, MFVec3f &normalArray) const
{
  TriangleFanSet *T = static_cast<TriangleFanSet *>(G->getLink());

  Coordinate *CoordNode = static_cast<Coordinate *>(T->getCoord());
  if (CoordNode == 0)
    return;
  MFVec3f vertexArray = CoordNode->getPoint();

  SFBool normalPerVertex = T->getNormalPerVertex();
  MFInt32 fanCount = T->getFanCount();
  
  //computing the array containing the normals of each face
  MFVec3f normalFaceArray;
  
  MFVec3f::const_iterator itCoord = vertexArray.begin();
  for (MFInt32::const_iterator itFan = fanCount.begin(); itFan != fanCount.end(); ++itFan)
  {
    SFVec3f Center = *itCoord;
    ++itCoord;
    SFVec3f A = *itCoord;
    ++itCoord;

    for (SFInt32 i = 2; i < *itFan; ++i) 
    {
      SFVec3f B = *itCoord;
      ++itCoord;
    
      //computing the normal following ccw
      SFVec3f AB = B - Center;
      SFVec3f AC = Center - A;
      SFVec3f N;
      if (G->getCcw())
        N = crossprod(AB, AC);
      else
        N = crossprod(AC, AB);  
      normalFaceArray.push_back(N);
    }
  }
  
  if (normalPerVertex)
  {
    normalArray = MFVec3f(vertexArray.size());
    
    //iterate the fans
    MFVec3f::iterator itNormal = normalArray.begin();
    MFVec3f::const_iterator itFace = normalFaceArray.begin();
    for (MFInt32::const_iterator itFan = fanCount.begin(); itFan != fanCount.end(); ++itFan)
    {
      SFVec3f Center = *itNormal; //memorizing the center of the fan
      ++itNormal;//positioning on the second point
      SFVec3f N0 = *itFace;
      Center = Center + N0;
      ++itFace;
      *itNormal = N0;//the first point has the normal of the first face
      ++itNormal;
      for (SFInt32 i = 2; i < *itFan; ++i) 
      {
        SFVec3f N1 = *itFace;
        Center = Center + N1;
        SFVec3f N = N0 + N1;
        N.normalize();
        *itNormal = N;
        N0 = N1;
        ++itNormal;
      }
      Center.normalize();//normalizing the center of the fan
    }
  }
  else
  {
    //copying the values of normalFaceArray
    for (MFVec3f::const_iterator itFace = normalFaceArray.begin(); itFace != normalFaceArray.end(); ++itFace)
      normalArray.push_back(*itFace);
  }
}

void GLUpdaterGLRenderingVisitor::computeNoColor_NoTexCoord(GLTriangleSet *G, const MFVec3f &normalArray) const
{
  TriangleSet *T = static_cast<TriangleSet *>(G->getLink());

  Coordinate *CoordNode = static_cast<Coordinate *>(T->getCoord());
  if (CoordNode == 0)
    return;
  MFVec3f vertexArray = CoordNode->getPoint();

  SFBool normalPerVertex = T->getNormalPerVertex();
  if (normalPerVertex)
  {
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    //iterate the coords
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++ itCoord)
    {
      N3F_V3F element;  
      element.normal = *itNormal;
      element.vertex = *itCoord;
      G->getN3F_V3FvertexTriangleArray().push_back(element);
            
      ++itNormal;
    }
  }
  else //normalPerVertex is FALSE
  {
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    //iterate the coords
    unsigned int vertexCount = 0;
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++ itCoord)
    {
      N3F_V3F element;  
      element.normal = *itNormal;
      element.vertex = *itCoord;
      G->getN3F_V3FvertexTriangleArray().push_back(element);
        
      if ((vertexCount + 1)%3 == 0)      
        ++itNormal;
    }
  }
}

void GLUpdaterGLRenderingVisitor::computeColor_NoTexCoord(GLTriangleSet *G, const MFVec3f &normalArray) const
{
  TriangleSet *T = static_cast<TriangleSet *>(G->getLink());

  MFColorRGBA colorRGBA = static_cast<ColorRGBA *>(T->getColor())->getColor();
  
  Coordinate *CoordNode = static_cast<Coordinate *>(T->getCoord());
  if (CoordNode == 0)
    return;
  MFVec3f vertexArray = CoordNode->getPoint();

  SFBool normalPerVertex = T->getNormalPerVertex();
  SFBool colorPerVertex = T->getColorPerVertex();
      
  if ((normalPerVertex) && (colorPerVertex))
  {
    MFColorRGBA::const_iterator itColor = colorRGBA.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    //iterate the coords
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++itCoord)
    {
      C4F_N3F_V3F element;  
      element.color = *itColor;
      element.normal = *itNormal;
      element.vertex = *itCoord;
      G->getC4F_N3F_V3FvertexTriangleArray().push_back(element);
            
      ++itColor;
      ++itNormal;
    }
  }
  else //normalPerVertex or colorPertex is FALSE
  {
    MFColorRGBA::const_iterator itColor = colorRGBA.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    //iterate the coords
    unsigned int vertexCount = 0;
    for (MFVec3f::const_iterator itCoord = vertexArray.begin(); itCoord != vertexArray.end(); ++ itCoord)
    {
      C4F_N3F_V3F element;  
      element.color = *itColor;
      element.normal = *itNormal;
      element.vertex = *itCoord;
      G->getC4F_N3F_V3FvertexTriangleArray().push_back(element);
        
      if ((colorPerVertex) || ((vertexCount)%3 == 2))    
        ++itColor;
      if ((normalPerVertex) || ((vertexCount)%3 == 2))      
        ++itNormal;
          
      vertexCount++;  
    }
  }  
}

void GLUpdaterGLRenderingVisitor::computeNoColor_TexCoord(GLTriangleSet *G, const MFVec3f &normalArray) const
{
}

void GLUpdaterGLRenderingVisitor::computeColor_TexCoord(GLTriangleSet *G, const MFVec3f &normalArray) const
{
}

void GLUpdaterGLRenderingVisitor::computeNormalArray(GLTriangleSet *G, MFVec3f &normalArray) const
{
  TriangleSet *T = static_cast<TriangleSet *>(G->getLink());

  Coordinate *CoordNode = static_cast<Coordinate *>(T->getCoord());
  if (CoordNode == 0)
    return;
  MFVec3f vertexArray = CoordNode->getPoint();

  SFBool normalPerVertex = T->getNormalPerVertex();
    
  //computing the array containing the normals of each face
  MFVec3f normalFaceArray;
  
  MFVec3f::const_iterator itCoord = vertexArray.begin();
  while (itCoord != vertexArray.end())
  {
    SFVec3f A = *itCoord;
    ++itCoord;
    SFVec3f B = *itCoord;
    ++itCoord;
    SFVec3f C = *itCoord;
    ++itCoord;
    
    SFVec3f AB = B - A;
    SFVec3f AC = C - A;
    SFVec3f N;
    if (G->getCcw())
      N = crossprod(AB, AC);
    else
      N = crossprod(AC, AB);  
    normalFaceArray.push_back(N);
  }
  
  if (normalPerVertex)
  {
    normalArray = MFVec3f(vertexArray.size());
     
    MFVec3f::iterator itNormal = normalArray.begin();
    for (MFVec3f::const_iterator itFace = normalFaceArray.begin(); itFace != normalFaceArray.end(); ++itFace)
    {
      *itNormal = *itFace;
      ++itNormal;
      *itNormal = *itFace;
      ++itNormal;
      *itNormal = *itFace;
      ++itNormal;
    }
  }
  else
  {
    //copying the values of normalFaceArray
    for (MFVec3f::const_iterator itFace = normalFaceArray.begin(); itFace != normalFaceArray.end(); ++itFace)
      normalArray.push_back(*itFace);
  }
}

void GLUpdaterGLRenderingVisitor::computeNoColor_NoTexCoord(GLTriangleStripSet *G, const MFVec3f &normalArray) const
{
  TriangleStripSet *T = static_cast<TriangleStripSet *>(G->getLink());

  Coordinate *CoordNode = static_cast<Coordinate *>(T->getCoord());
  if (CoordNode == 0)
    return;
  MFVec3f vertexArray = CoordNode->getPoint();

  SFBool normalPerVertex = T->getNormalPerVertex();
  MFInt32 stripCount = T->getStripCount();
    
  if (normalPerVertex)
  {
    G->getVerticesDuplicated() = false;
    //using the GL_N3F_V3F
    if (normalPerVertex)
    {
      //using the GL_TRIANGLE_STRIP
      MFVec3f::const_iterator itCoord = vertexArray.begin();
      MFVec3f::const_iterator itNormal = normalArray.begin();
        
      //iterate the strips
      for (MFInt32::const_iterator itStrip = stripCount.begin(); itStrip != stripCount.end(); ++ itStrip)
      {
        //filling a new array
        vector<N3F_V3F> *vertexStripArray = new vector<N3F_V3F>();
        for (int i = 0; i < *itStrip; ++i)
        {
          N3F_V3F element;  
          element.normal = *itNormal;
          element.vertex = *itCoord;
          vertexStripArray->push_back(element);
            
          ++itCoord;
          ++itNormal;
        }
        G->getN3F_V3FvertexStripArrayArray().push_back(vertexStripArray);
      }
    } 
  }
  else //normalPerVertex is FALSE
  {
    G->getVerticesDuplicated() = true;
    //using the GL_TRIANGLES
    MFVec3f::const_iterator itCoord = vertexArray.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    //iterate the strips
    for (MFInt32::const_iterator itStrip = stripCount.begin(); itStrip != stripCount.end(); ++ itStrip)
    {
      //filling a new array
      vector<N3F_V3F> *vertexStripArray = new vector<N3F_V3F>();
      N3F_V3F center;
      center.normal = *itNormal;
      center.vertex = *itCoord;
      ++itCoord;
        
      N3F_V3F element;  
      element.normal = *itNormal;
      element.vertex = *itCoord;

      for (int i = 2; i < *itStrip; ++i)
      {
        //push center
        center.normal = *itNormal;
        vertexStripArray->push_back(center);
 
        //push Pi
        element.normal = *itNormal;
        vertexStripArray->push_back(element);
        ++itCoord;
         
        //push Pi+1  
        element.normal = *itNormal;
        element.vertex = *itCoord;
        vertexStripArray->push_back(element);
        ++itNormal;  
      }
      G->getN3F_V3FvertexStripArrayArray().push_back(vertexStripArray);
    }
  } 
}

void GLUpdaterGLRenderingVisitor::computeColor_NoTexCoord(GLTriangleStripSet *G, const MFVec3f &normalArray) const
{
  TriangleStripSet *T = static_cast<TriangleStripSet *>(G->getLink());

  MFColorRGBA colorRGBA = static_cast<ColorRGBA *>(T->getColor())->getColor();
  
  Coordinate *CoordNode = static_cast<Coordinate *>(T->getCoord());
  if (CoordNode == 0)
    return;
  MFVec3f vertexArray = CoordNode->getPoint();

  SFBool normalPerVertex = T->getNormalPerVertex();
  SFBool colorPerVertex = T->getColorPerVertex();
  MFInt32 stripCount = T->getStripCount();
    
  if ((normalPerVertex) && (colorPerVertex))
  {
    G->getVerticesDuplicated() = false;
    //using the GL_TRIANGLE_STRIP
    MFColorRGBA::const_iterator itColor = colorRGBA.begin();
    MFVec3f::const_iterator itCoord = vertexArray.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    //iterate the strips
    for (MFInt32::const_iterator itStrip = stripCount.begin(); itStrip != stripCount.end(); ++ itStrip)
    {
      //filling a new array
      vector<C4F_N3F_V3F> *vertexStripArray = new vector<C4F_N3F_V3F>();
      for (SFInt32 i = 0; i < *itStrip; ++i)
      {
        C4F_N3F_V3F element;  
        element.color = *itColor;
        element.normal = *itNormal;
        element.vertex = *itCoord;
        vertexStripArray->push_back(element);
            
        ++itColor;
        ++itCoord;
        ++itNormal;
      }
      G->getC4F_N3F_V3FvertexStripArrayArray().push_back(vertexStripArray);
    }
  }
  else //normalPerVertex or colorPertex is FALSE
  {
    G->getVerticesDuplicated() = true;
    //using the GL_TRIANGLES
    MFColorRGBA::const_iterator itColor = colorRGBA.begin();
    MFVec3f::const_iterator itCoord = vertexArray.begin();
    MFVec3f::const_iterator itNormal = normalArray.begin();
        
    //iterate the strips
    bool orientation = true; //var used to change the order of the vertices in the
    //case of culling
    for (MFInt32::const_iterator itStrip = stripCount.begin(); itStrip != stripCount.end(); ++ itStrip)
    {
      //filling a new array
      vector<C4F_N3F_V3F> *vertexStripArray = new vector<C4F_N3F_V3F>();
      C4F_N3F_V3F P0;
      P0.color = *itColor;
      P0.normal = *itNormal;
      P0.vertex = *itCoord;
 
      ++itCoord;
      if (colorPerVertex)
        ++itColor;
      if (normalPerVertex)
        ++itNormal;  

      C4F_N3F_V3F P1, P2;  
      P1.color = *itColor;
      P1.normal = *itNormal;
      P1.vertex = *itCoord;

      for (SFInt32 i = 2; i < *itStrip; ++i)
      {
        //push center
        if (!colorPerVertex)
          P0.color = *itColor;
        if (!normalPerVertex)
          P0.normal = *itNormal;
        vertexStripArray->push_back(P0);
 
        //push Pi
        if (!colorPerVertex)
          P1.color = *itColor;
        if (!normalPerVertex)  
          P1.normal = *itNormal;
          
          
        ++itCoord;
        if (colorPerVertex)
          ++itColor;
        if (normalPerVertex)
          ++itNormal;  
            
        //push Pi+1
        P2.color = *itColor;
        P2.normal = *itNormal;
        P2.vertex = *itCoord;
         
        if (orientation)
        {
          vertexStripArray->push_back(P1);
          vertexStripArray->push_back(P2);
        }
        else
        {
          vertexStripArray->push_back(P2);
          vertexStripArray->push_back(P1);
        }  
        orientation = !orientation;
            
        if (!colorPerVertex)
          ++itColor;
        if (!normalPerVertex)
          ++itNormal;  
         
        //P0 becomes P1, P1 P2  
        P0 = P1;  
        P1 = P2;
      }
      G->getC4F_N3F_V3FvertexStripArrayArray().push_back(vertexStripArray);
    }
  }  
}

void GLUpdaterGLRenderingVisitor::computeNoColor_TexCoord(GLTriangleStripSet *G, const MFVec3f &normalArray) const
{
}

void GLUpdaterGLRenderingVisitor::computeColor_TexCoord(GLTriangleStripSet *G, const MFVec3f &normalArray) const
{
}

void GLUpdaterGLRenderingVisitor::computeNormalArray(GLTriangleStripSet *G, MFVec3f &normalArray) const
{
  TriangleStripSet *T = static_cast<TriangleStripSet *>(G->getLink());

  Coordinate *CoordNode = static_cast<Coordinate *>(T->getCoord());
  if (CoordNode == 0)
    return;
  MFVec3f vertexArray = CoordNode->getPoint();

  SFBool normalPerVertex = T->getNormalPerVertex();
  MFInt32 stripCount = T->getStripCount();
  
  //computing the array containing the normals of each face
  MFVec3f normalFaceArray;
  bool orientation = true;
  MFVec3f::const_iterator itCoord = vertexArray.begin();
  for (MFInt32::const_iterator itStrip = stripCount.begin(); itStrip != stripCount.end(); ++itStrip)
  {
    SFVec3f A = *itCoord;
    ++itCoord;
    SFVec3f B = *itCoord;
    ++itCoord;

    for (SFInt32 i = 2; i < *itStrip; ++i) 
    {
      SFVec3f C = *itCoord;
      ++itCoord;
    
      //computing the normal following ccw
      SFVec3f AB = B - A;
      SFVec3f AC = C - A;
      SFVec3f N;
      if (G->getCcw())
      {
        if (orientation)
          N = crossprod(AB, AC);
        else
          N = crossprod(AC, AB);    
      }   
      else
      {
        if (orientation)
          N = crossprod(AC, AB);              
        else
          N = crossprod(AB, AC);
      }
      normalFaceArray.push_back(N);
      A = B;
      B = C;
    }
  }
  
  if (normalPerVertex)
  {
    normalArray = MFVec3f(vertexArray.size());
    
    //iterate the fans
    MFVec3f::iterator itNormal = normalArray.begin();
    MFVec3f::const_iterator itFace = normalFaceArray.begin();
    for (MFInt32::const_iterator itStrip = stripCount.begin(); itStrip != stripCount.end(); ++itStrip)
    {
      SFVec3f N0 = *itFace;
      ++itFace;
      *itNormal = N0; 
      ++itNormal;
      for (SFInt32 i = 2; i < *itStrip; ++i) 
      {
        SFVec3f N1 = *itFace;
        *itNormal = N0 + N1;
        itNormal->normalize();
        N0 = N1;
        ++itNormal;
        ++itFace;
      }
    }
  }
  else
  {
    //copying the values of normalFaceArray
    for (MFVec3f::const_iterator itFace = normalFaceArray.begin(); itFace != normalFaceArray.end(); ++itFace)
      normalArray.push_back(*itFace);
  }
}

