#include "GLUpdaterGLTexturingVisitor.h"
#include "GLImageTexture.h"
#include "ImageTexture.h"

#include <math.h>
#include <GL/gl.h>
#include <qimage.h>
#include <qgl.h>
#include <qstring.h>

#include <iostream>

using namespace X3DTK;
using namespace std;

GLUpdaterGLTexturingVisitor::GLUpdaterGLTexturingVisitor()
: GLTexturingVisitor()
{
  // Enter functions.
  defineNewEnterFunction<GLUpdaterGLTexturingVisitor, GLImageTexture>(&GLUpdaterGLTexturingVisitor::enterGLImageTexture);


  // GlobalVariables assignation.
  globalVariables = GVManager::getInstanceOf<GLUpdaterGlobalVariables>();
}

GLUpdaterGLTexturingVisitor::~GLUpdaterGLTexturingVisitor()
{
}

void GLUpdaterGLTexturingVisitor::enterGLImageTexture(GLImageTexture *G) const
{
  ImageTexture *I = static_cast<ImageTexture *>(G->getLink());
  
  MFString url = I->getUrl();
  if (url.empty())
    return;
  
  const char *file = SFString(globalVariables->getPath() + "/" + url.front());
  
  QImage img(file);
  
  if (img.isNull())
  {
    cerr << "unable to load the texture from " << file << ", unsupported file format" << endl;
    return;
  }
  // 1E-3 needed. Just try with width=128 and see !
  int newWidth  = 1<<(int)(1+log(img.width() -1+1E-3) / log(2.0));
  int newHeight = 1<<(int)(1+log(img.height()-1+1E-3) / log(2.0));

  //set the ratios for the next geometry node
  SFFloat ratioS = (float)(img.width())/(float)(newWidth);
  SFFloat ratioT = (float)(img.height())/(float)(newHeight);
  
  G->getTextureTransform()[0] = ratioS;
  G->getTextureTransform()[12] = -ratioS + 2.0f;
  
  G->getTextureTransform()[5] = ratioT;
  G->getTextureTransform()[13] = 1.0f - ratioT;
        
  if ((img.width() != newWidth) || (img.height() != newHeight))
    img = img.copy(0, 0, newWidth, newHeight);

  QImage glImg = QGLWidget::convertToGLFormat(img);  // flipped 32bit RGBA

  // GL commands
  glGenTextures(1, &G->getTexName());
  
  glBindTexture(GL_TEXTURE_2D, G->getTexName());
  
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);

  if (I->getRepeatS())
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  else
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
    
  if (I->getRepeatT())
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  else
    glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
    
  glTexImage2D(GL_TEXTURE_2D, 0, 3, glImg.width(), glImg.height(), 0, GL_RGBA, GL_UNSIGNED_BYTE, glImg.bits());
}

