#include "SceneToFile.h"
#include "DefaultDFS.h"
#include "SceneToFileCoreVisitor.h"

#include <fstream>
#include <iostream>

using namespace X3DTK;
using namespace std;

SceneToFile::SceneToFile(DFS *dfs)
: X3DSceneToFile()
{
  globalVariables = GVManager::getInstanceOf<SceneToFileGlobalVariables>();
  
  if (dfs == 0)
  {
    dfs = new DefaultDFS();
    defaultDFSUSE = true;
  }  
  else
  {
    this->dfs = dfs;
    defaultDFSUSE = false;
  }
  
  dfs->setComponentVisitor(new SceneToFileCoreVisitor());
}

SceneToFile::~SceneToFile()
{
  if (defaultDFSUSE)
    delete dfs;
}

void SceneToFile::setComponentVisitor(X3DComponentVisitor *component)
{
  dfs->setComponentVisitor(component);
}

void SceneToFile::toFile(Scene *S, const SFString &file) const
{
  ofstream output(file);
  
  output << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;
  output << "<X3D>" << endl;
  dfs->traverse(S);
  MFString text = globalVariables->getOutput();
  
  for (MFString::const_iterator it = text.begin(); it != text.end(); ++it)
    output << (*it) << endl;

  output << "</X3D>" << endl;
}
