/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            SceneToFileGlobalVariables.h                    //
////////////////////////////////////////////////////////////////////////////////

#ifndef SCENETOFILEGLOBALVARIABLES_H
#define SCENETOFILEGLOBALVARIABLES_H

#include "GlobalVariables.h"
#include "NameDict.h"

#include <list>

namespace X3DTK {

/// GlobalVariables for the SceneToFile module.

class SceneToFileGlobalVariables : public GlobalVariables
{
public:
  /// Constructor.
  SceneToFileGlobalVariables();
  /// Destructor.
  virtual ~SceneToFileGlobalVariables();
  
  /// Sets the hasUSE variable.
  void pushHasUSE(bool value);
  
  /// New line.
  void newLine();
  /// Pushes line.
  void pushCurrentLine();
  /// Writes the string.
  void writeCurrentLine(const SFString &string);
  /// Writes the string.
  void writeOpeningLine(const SFString &string);

  /// Adds a tabulation.
  void addTab();
  /// Removes a tabulation.
  void removeTab();
  
  /// Gets the SFString of the output.
  inline MFString getOutput() const {return _output;};
  /// Returns TRUE if the current visited node has a child.
  bool hasChild();
  /// Pops the line stack.
  void popLine();
  /// Returns the hasUSE variable.
  inline bool getHasUSE() const {return _hasUSEStack.front();};
  /// Pops the hasUSE stack.
  void popHasUSE();
  /// Gets the tabulation.
  inline SFString getTab() const {return _tab;};
  /// returns the DEF attribute of the node.
  SFString getDEF(const SFNode &N);
  /// returns the USE attribute of the node.
  SFString getUSE(const SFNode &N) const;
  
private:
  int _lineNumber;
  MFString _output; 
  SFString _tab;
  std::list<int> _lineStack;
  std::list<bool> _hasUSEStack;
  std::list<SFString> _DEFList;
};

}

#endif
