#include "SimpleX3DGLScene.h"
#include "DefaultLoader.h"
#include "BboxUpdater.h"
#include "GLConverter.h"
#include "GLUpdater.h"
#include "GLRenderer.h"
#include "MemReleaser.h"
#include "Scene.h"
#include "GLScene.h"
#include "Bbox.h"

#include <iostream>
#include <time.h>

using namespace std;
using namespace X3DTK;

SimpleX3DGLScene::SimpleX3DGLScene()
: s(0), gs(0)
{
  _loader = new DefaultLoader();
  _bboxupdater = new BboxUpdater();
  _converter = new GLConverter();
  _renderer = new GLRenderer();
  _releaser = new MemReleaser();
}

SimpleX3DGLScene::~SimpleX3DGLScene()
{
  _releaser->release(s);
  _releaser->release(gs);  
  s = 0;
  gs = 0;

  delete _loader;
  delete _bboxupdater;
  delete _converter;
  delete _renderer;
  delete _releaser;
}

void SimpleX3DGLScene::setLoader(X3DLoader *loader)
{
  _loader = loader;
}

void SimpleX3DGLScene::setBboxUpdater(X3DBboxUpdater *bboxUpdater)
{
  _bboxupdater = bboxUpdater;
}

void SimpleX3DGLScene::setGLCreator(X3DGLCreator *creator)
{
  _converter->setGLCreator(creator);
}

void SimpleX3DGLScene::setGLUpdater(X3DGLUpdater *updater)
{
  _converter->setGLUpdater(updater);
}

void SimpleX3DGLScene::setGLRenderer(X3DGLRenderer *renderer)
{
  _renderer = renderer;
}

void SimpleX3DGLScene::setMemReleaser(X3DMemReleaser *memReleaser)
{
  _releaser = memReleaser;
}
  
void SimpleX3DGLScene::load(const char *file)
{
  _releaser->release(s);
  _releaser->release(gs);  

  cout << "loading..." << endl;
  clock_t begin = clock();
  
  s = _loader->load(file);
  
  clock_t end = clock();
  cout << "loaded in " << (end - begin)/1000 << endl;
 
  _bboxupdater->update(s, true);

  if (s != 0)
  {
    SFVec3f center = s->getBboxCenter();
    SFVec3f size = s->getBboxSize();

    SFVec3f A = center + 0.5f*size;
    SFVec3f B = center - 0.5f*size;

    if (A.x < B.x)
    {
      min.x = A.x;
      max.x = B.x;
    }  
    else
    {
      min.x = B.x;  
      max.x = A.x;
    }  
  
    if (A.y < B.y)
    {
      min.y = A.y;
      max.y = B.y;
    }  
    else
    {
      min.y = B.y;  
      max.y = A.y;
    }
  
    if (A.z < B.z)
    {
      min.z = A.z;
      max.z = B.z;
    }  
    else
    {
      min.z = B.z;  
      max.z = A.z;
    }  
   
    gs = _converter->convert(s);
  }
}

void SimpleX3DGLScene::draw()
{
  _renderer->render(gs);
}

void SimpleX3DGLScene::release()
{
  _releaser->release(s);
  _releaser->release(gs);
  s = 0;
  gs = 0;  
}
