/// This file is part of the QGLViewer library
/// Copyright (C) 2002-2003  Yannick Le Goc (legoc@imag.fr)

/// http://www-imagis.imag.fr/Membres/Yannick.Legoc/X3D/


/// X3DToolKit is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// X3DToolKit is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with X3DToolKit; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA



/// Ce programme est libre, vous pouvez le redistribuer et/ou le modifier
/// selon les termes de la Licence Publique Gnrale GNU publie par la Free
/// Software Foundation (version 2 ou bien toute autre version ultrieure
/// choisie par vous).

/// Ce programme est distribu car potentiellement utile, mais SANS AUCUNE
/// GARANTIE, ni explicite ni implicite, y compris les garanties de
/// commercialisation ou d'adaptation dans un but spcifique. Reportez-vous
///  la Licence Publique Gnrale GNU pour plus de dtails.

/// Vous devez avoir reu une copie de la Licence Publique Gnrale GNU en
/// mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free
/// Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
/// 02111-1307, tats-Unis. 

////////////////////////////////////////////////////////////////////////////////
//                            SimpleX3DGLScene.h                              //
////////////////////////////////////////////////////////////////////////////////

#ifndef SIMPLEX3DGLSCENE_H
#define SIMPLEX3DGLSCENE_H

#include "X3DTypes.h"
#include "Bbox.h"

namespace X3DTK {

class X3DLoader;
class X3DBboxUpdater;
class GLConverter;
class X3DGLCreator;
class X3DGLUpdater;
class X3DGLRenderer;
class X3DMemReleaser;
class Scene;
class GLScene;

/// Class defining a facade to allow an easy load and display of an X3D scene.

class SimpleX3DGLScene
{
public:
  /// Constructor.
  SimpleX3DGLScene();
  /// Destructor.
  virtual ~SimpleX3DGLScene();
  
  /// Loads ths scene from file.
  void load(const char *file);
  /// Draw the scene in an opengl context.
  virtual void draw();
  /// Gets the Bounding Box center.
  inline SFVec3f getBboxMin() const {return min;};
  /// Gets the Bounding Box size.
  inline SFVec3f getBboxMax() const {return max;};
  /// Releases the memory.
  void release();

  /// Sets the loader.
  void setLoader(X3DLoader *loader);
  /// Sets the BboxUpdater.
  void setBboxUpdater(X3DBboxUpdater *bboxUpdater);  
  /// Sets the GLCreator.
  void setGLCreator(X3DGLCreator *creator);  
  /// Sets the GLUpdater.
  void setGLUpdater(X3DGLUpdater *glupdater);  
  /// Sets the GLRenderer.
  void setGLRenderer(X3DGLRenderer *renderer);  
  /// Sets the MemReleaser.
  void setMemReleaser(X3DMemReleaser *memReleaser);  
  
protected:
  X3DLoader *_loader;
  X3DBboxUpdater *_bboxupdater;
  GLConverter *_converter;
  X3DGLRenderer *_renderer;
  X3DMemReleaser *_releaser;
  Scene *s;
  GLScene *gs;
  SFVec3f min;
  SFVec3f max;
};

}

#endif
